function [cImage mImageResized] = BlockMImage(mImage, blockHeight, blockWidth)

% Splits a matrix into cells of nRowsxnColumns

if nargin < 3
    blockWidth = blockHeight;
end

% Get image dimensions
% < < - - #
    [y x z] = size(mImage);
% - - > > #

% Check if it can be split exactly into these dimensions
% < < - - #
if mod(y,blockHeight) || mod(x,blockWidth);
    
    fprintf('A %ix%i image cannot be split exactly into blocks of %i x %i,\nResizing.........\n',...
        y,x,blockHeight,blockWidth);
    nRowBlocks =floor(size(mImage,1)/blockHeight);
    nColumnBlocks =floor(size(mImage,2)/blockWidth);
    nRows = nRowBlocks*blockHeight;
    nColumns = nColumnBlocks*blockWidth;
    if nRows == 0 || nColumns == 0
        error('Image is too small to be split into square blocks!!!!');
    end
    mImageResized = mImage(1:nRows,1:nColumns,:);
    fprintf('New image is  %ix%i pixels.\n',nRows,nColumns);
    
else
    
    mImageResized = mImage;
    nRows = y;
    nColumns = x;
    nRowBlocks = nRows/blockHeight;
    nColumnBlocks = nColumns/blockWidth;
    
end

% Split the image into cells
% < < - - #
    if z == 1
        cImage = mat2cell(mImageResized, blockHeight*ones(1, nRowBlocks), ...
            blockWidth*ones(1, nColumnBlocks));
    elseif z == 3
        % Quicker to use Mat2Cell for fixed size cells       
        cImage = Mat2Cell(mImageResized,blockHeight,blockWidth);
%         cImage = mat2cell(mImageResized, blockHeight*ones(1, nRowBlocks),...
%             blockWidth*ones(1, nColumnBlocks),3);        
        
    end
% - - > > #

