function mImageRecon = ExpandImageGrey(mImageFolded,cIndex,nCoefficients,...
    sApproxDetails,sBlockDims, publicKey, privateKey, rotate,sTuning)

% PrintWithStars([mfilename ,' Routine Called']);
%% Input validation
% Check mandatory inputs
if nargin < 5
    fprintf('Image matrix, cofficient indices, method and blockWidth are mandatory\n');
    return;
end

% Now check optional arguments
if nargin < 6
    publicKey = 19;
end
if nargin < 7
    privateKey = 987654321;
end
if nargin < 8
    rotate = 1;
end
if nargin < 9
    sTuning.vTuningFactor = 1;
    sTuning.quantMethod = 'none';
    sTuning.scaleEmbeddedImage = 0;
    sTuning.displayAnalysis = 0;
end

% Show some info
% < < - - #
    showInfo = true;
% - - > > #

% Set up the key structure
    sKey.publicKey = publicKey;
    sKey.privateKey = privateKey;
    sKey.rotate = rotate;
% - - > > #

% Initialize the folded image structure
    sImageFolded = SImageFolded;
    [sImageFolded.y sImageFolded.x sImageFolded.z] = size(mImageFolded);
% - - > > #   

% Block the image matrix
% < < - - #
    sImageFolded.cImage = BlockMImage(mImageFolded,sBlockDims.approxHeight...
        ,sBlockDims.approxWidth);
    sImageFolded.blockWidth = sBlockDims.approxWidth;
    sImageFolded.blockHeight = sBlockDims.approxHeight;
    [sImageFolded.nYBlocks sImageFolded.nXBlocks] = ...
        size(sImageFolded.cImage);
    sImageFolded.nPixelsBlock = sImageFolded.blockWidth * sImageFolded.blockHeight;
% - - > > #

% Set up the recovered image structrue
    sImageRecon = SImageRecon;
    sImageRecon.nCoefficients = nCoefficients;
    sImageRecon.cIndex = cIndex;
    [sImageRecon.nYBlocks sImageRecon.nXBlocks] = size(sImageRecon.cIndex);
    sImageRecon.nBlocks = sImageRecon.nYBlocks*sImageRecon.nXBlocks;
    sImageRecon.nPixelsBlock = sImageFolded.nPixelsBlock;
    sImageRecon.blockWidth = sBlockDims.approxWidth;
    sImageRecon.blockHeight = sBlockDims.approxHeight;
% - - > > #
    
% Calculate position of last folded block, first recon block and length of
% padding.
    [iXBlockEnd iXBlockStart iYBlockStart lPadding] = LastBlock(...
        sImageRecon,sImageFolded.nYBlocks);    
% - - > > #

% Initialize the dictionary
    sDictionary = Dictionary(sApproxDetails.sDetails,sImageRecon.blockWidth);
    mDictionary2D = kron(sDictionary.mDictionary,sDictionary.mDictionary);
% - - > > #

% Seperate the host and hidden coefficients
    start = clock;
    [ sImageRecon, vRecoveredCn] = SeparateCellImage(sImageFolded,sImageRecon,...
        mDictionary2D,iXBlockEnd,sKey);
    removeTime = etime(clock,start);
    
    if showInfo
        fprintf('Time to seperate the embedded image (F) from the host image: %.2f seconds\n',...
            removeTime);
    end
    
    % Remove zero padding
        vRecoveredCn = vRecoveredCn(1:end-lPadding);
    % - - > > #
% - - > > #

% Reorder the coefficients
%     vRecoveredCn = RRandPerm(vRecoveredCn,sKey.publicKey);
% - - > > #

% Recostruct the rest of the image from hidden coefficients
    start = clock;
    sImageRecon = ReconstructCellImage(sImageRecon, vRecoveredCn,...
        mDictionary2D, iYBlockStart, iXBlockStart);
    reconstructTime = etime(clock,start);
    if showInfo
        fprintf('Time to reconstruct the rest of the image: %.2f seconds\n',...
            reconstructTime);
    end
% - - > > #    
    
% Put recovered image into a matrix
    mImageRecon = cell2mat(sImageRecon.cImage);
% - - > > #
%% Block the image
% nXBlocks will be the same as in the original image nYBlocks would be determined 
% from the number of block separators in the bit stream.
% [cFoldedImage, mFoldedImage , nYBlocksFolded, nXBlocks] = BlockImage(mFoldedImage, blockWidth);
% nPixelsBlock = blockWidth^2;
%% Calculate size parameters from the cIndex which is calculated from the bitstream
% Here the structure cIndex represents the information we get from the
% bitstream.  This simplifies the situation so that when a bitstream is
% supplied we can just populate cIndex without having to change the rest of
% the code.
%{
nCoefficients = GetNCn(cIndex);
nBlocks = numel(cIndex);
nBlocksUsed = ceil(nCoefficients/nPixelsBlock);
lFree = nBlocksUsed*nPixelsBlock - nCoefficients;
iXBlockEnd = nBlocksUsed - ((nYBlocksFolded-1)*nXBlocks);
nYBlocks = nBlocks/nXBlocks;
if iXBlockEnd == nXBlocks
    iXBlockStart = 1;
    iYBlockStart = nYBlocksFolded+1;
else
    iXBlockStart = iXBlockEnd+1;
    iYBlockStart = nYBlocksFolded;
end
%}
% cRecoveredImage = cell(nYBlocks,nXBlocks);
%{
%% Seperate the host and hidden images
% Set up the dictionary here to avoid calculating it for every block
start = clock;
sDictionary = Dictionary(sApproxDetails.sDetails,blockWidth,-1);
mDictionary2D = kron(sDictionary.mDictionary,sDictionary.mDictionary);
[ cRecoveredImage, cEmbeddedRecovery, vRecoveredCn] = ...
    SeparateCellImage(cFoldedImage, cRecoveredImage, mDictionary2D, ...
    blockWidth, iXBlockEnd, cIndex, sKey);
% Rescale the embedded image
if sTuning.scaleEmbeddedImage
    mEmbeddedImageRecovered = cell2mat(cEmbeddedRecovery)/alpha;
    cEmbeddedRecovery = BlockImage(mEmbeddedImageRecovered, blockWidth);
    vRecoveredCn = GetCnFromEmbeddedImage(cEmbeddedRecovery, mDictionary2D, blockWidth, iXBlockEnd, cIndex, sKey);
end
removeTime = etime(clock,start);
fprintf('Time to seperate the embedded image (F) from the host image: %.2f seconds\n',removeTime);
%{
Remove lFree padding from  vHiddenCoefficients.  If we were not
using an orthogonal basis to hide the coefficients then we would
distribute this space over all the blocks to avoid the calculation of the
projector being ill posed, as the proctor will be orthogonal this step is
unecessary.
%}
vRecoveredCn = vRecoveredCn(1:end-lFree);

% vRecoveredCn = vRecoveredCn(1:sImageFolded.nCoefficients);
%}
%{
%% Reverse the quantisation
if ~strcmp(sTuning.quantMethod,'none')
    cQuant = cIndex{1,1}.cQuant;
    % Get the original index's of all the hidden coefficients
    vHiddenIndex = GetFoldedCnIndex(cIndex, blockWidth, iYBlockStart, iXBlockStart);
    % Reverse the quantisation
    vRecoveredCn = RRescaleCoefficients(vRecoveredCn,vHiddenIndex,cQuant, sTuning.displayAnalysis);
end
%}
%{
%% Reconstruct the rest of the image using vHiddenCoefficients
start = clock;
cRecoveredImage = ReconstructCellImage(cRecoveredImage, vRecoveredCn,...
    mDictionary2D, cIndex, blockWidth, iYBlockStart, iXBlockStart);
reconstructTime = etime(clock,start);
fprintf('Time to reconstruct the rest of the image: %.2f seconds\n',reconstructTime);
mRecoveredImage = cell2mat(cRecoveredImage);
% if original image is provided compare with hidden to get PSNR
mHostImageRecovered = GetSubImage(cell2mat(cRecoveredImage),[1,1], [nYBlocksFolded, iXBlockEnd], blockWidth);
mHiddenImageRecovered = GetSubImage(cell2mat(cRecoveredImage),[iYBlockStart, iXBlockStart],[nYBlocks,nXBlocks], blockWidth);
if sTuning.displayAnalysis
    %% Display the results
    if compareApproximation
        mHostImage = GetSubImage(mApproxImage,[1,1], [nYBlocksFolded, iXBlockEnd], blockWidth);
        mHiddenImage = GetSubImage(mApproxImage,[iYBlockStart, iXBlockStart],[nYBlocks,nXBlocks], blockWidth);
        PrintWithStars('PSNR between approximation and recovery splits:');
        PSNRHost = CalcPSNR(mHostImage,mHostImageRecovered, maxIntensity);
        fprintf('Host image: %.4f dB\n',PSNRHost);
        PSNRHidden = CalcPSNR(mHiddenImage,mHiddenImageRecovered, maxIntensity);
        fprintf('Hidden image: %.4f dB\n',PSNRHidden);
    end
    if compareApproximation
        PSNRHostPlusHidden = CalcPSNR(mApproxImage,mRecoveredImage, maxIntensity);
        fprintf('Whole Image: %.4f dB\n',PSNRHostPlusHidden);
    end
    if compareEmbedded
        for i = iXBlockEnd+1:nXBlocks
            cEmbeddedRecovery{end,i} = zeros(blockWidth);
        end
        mEmbeddedImageRecovered = cell2mat(cEmbeddedRecovery);
        PSNREmbedded = CalcPSNR(mEmbeddedImage,mEmbeddedImageRecovered, maxIntensity);
        PrintWithStars('PSNR between F the embedded image (c.f eq. 9 in [1]) we add and the embedded image we recover:');
        fprintf('%.4f dB\n',PSNREmbedded);
    end

end
if compareOriginal
    PrintWithStars('PSNR between the Original image and the recovery');
    PSNROriginalRecovered = CalcPSNR(round(mImage),round(mRecoveredImage), maxIntensity);
    fprintf('%.4f dB\n',PSNROriginalRecovered);
end
%}
end

%% Begin sub functions



function [iXBlockEnd iXBlockStart iYBlockStart lPadding] = ...
    LastBlock(sImageRecon,nYBlocks)
    
    % Position of last block
    
    nBlocksUsed = ceil(sImageRecon.nCoefficients/sImageRecon.nPixelsBlock);
    lPadding = nBlocksUsed*sImageRecon.nPixelsBlock - sImageRecon.nCoefficients;
    iXBlockEnd = nBlocksUsed - ((nYBlocks-1)*sImageRecon.nXBlocks);
    if iXBlockEnd == sImageRecon.nXBlocks
        iXBlockStart = 1;
        iYBlockStart = nYBlocks+1;
    else
        iXBlockStart = iXBlockEnd+1;
        iYBlockStart = nYBlocks;
    end     

end

function sImageRecon = ReconstructCellImage(sImageRecon, vCn,...
    mDictionary,iYBlockStart, iXBlockStart)

    rowIncrement = 1;
    iCn  = 1;
    nXBlocks = sImageRecon.nXBlocks;
    
    for i = iYBlockStart:sImageRecon.nYBlocks

        for j = iXBlockStart:rowIncrement:nXBlocks

            % Construct the matix of atoms used to reconstruct this block
            iMDictionary = sImageRecon.cIndex{i,j}.iMDictionary2D;
            mImageSpace = mDictionary(:,iMDictionary);
            % Get the coefficients to recover this block
            nCn = size(iMDictionary,2);
            vCnBlock = vCn(iCn:iCn+nCn-1);
            iCn = iCn + nCn;

            % Recover the block
            sImageRecon.cImage{i,j} = zeros(sImageRecon.blockWidth);
            sImageRecon.cImage{i,j}(:) = mImageSpace*vCnBlock;

        end
        if i == iYBlockStart
            iXBlockStart = 1;
        end
        temp = iXBlockStart;
        iXBlockStart = nXBlocks;
        nXBlocks = temp;
        rowIncrement = rowIncrement * -1;

    end

end
function sImageRecon = ReconstructCellImageX(sImageRecon, vCn,...
    mDictionary,iYBlockStart, iXBlockStart)

    rowIncrement = 1;
    iCn  = 1;
    nXBlocks = sImageRecon.nXBlocks;
    sImageRecon.cCoeff = cell(size(sImageRecon.cImage));
    
    for i = iYBlockStart:sImageRecon.nYBlocks

        for j = iXBlockStart:rowIncrement:nXBlocks

            % Construct the matix of atoms used to reconstruct this block
            iMDictionary = sImageRecon.cIndex{i,j}.iMDictionary2D;
            mImageSpace = mDictionary(:,iMDictionary);
            % Get the coefficients to recover this block
            nCn = size(iMDictionary,2);
            vCnBlock = vCn(iCn:iCn+nCn-1);
            iCn = iCn + nCn;

            sImageRecon.cCoeff{i,j} = vCnBlock;
            % Recover the block
            sImageRecon.cImage{i,j} = zeros(sImageRecon.blockWidth);
            sImageRecon.cImage{i,j}(:) = mImageSpace*vCnBlock;

        end
        if i == iYBlockStart
            iXBlockStart = 1;
        end
        temp = iXBlockStart;
        iXBlockStart = nXBlocks;
        nXBlocks = temp;
        rowIncrement = rowIncrement * -1;

    end

end

function nCn = GetNCn(cIndex)

[nYBlocks, nXBlocks] = size(cIndex);
nCn = 0;
for i = 1:nYBlocks
    for j = 1:nXBlocks
        nCn = nCn + numel(cIndex{i,j}.iMDictionary2D);
    end
end

end

function vHiddenIndex = GetFoldedCnIndex(cIndex, blockWidth, iYBlockStart, iXBlockStart)

    [nYBlocks, nXBlocks] = size(cIndex);
    % Allocate memory for variables
    vHiddenIndex = zeros(blockWidth^2*nYBlocks*nXBlocks,1);
    rowIncrement = 1;
    iVHiddenIndex  = 1;
    for i = iYBlockStart:nYBlocks

        for j = iXBlockStart:rowIncrement:nXBlocks

            iMDictionary2D = cIndex{i,j}.iMDictionary2D;
            nCn = size(iMDictionary2D,2);
            vHiddenIndex(iVHiddenIndex:iVHiddenIndex + nCn -1) = iMDictionary2D;
            iVHiddenIndex = iVHiddenIndex + nCn;     

        end
        if i == iYBlockStart
            iXBlockStart = 1;
        end
        temp = iXBlockStart;
        iXBlockStart = nXBlocks;
        nXBlocks = temp;
        rowIncrement = rowIncrement * -1;

    end

end

function vHiddenR = ReorderCn(vHidden)

    % Reverse odd and even reordering enforced by folding routine.
    
    % Establish dimensions
        nHidden = numel(vHidden);

        % If odd one more odd indexes coefficients than even
        if mod(nHidden,2)
            nOdd = (nHidden+1)/2;
        % Otherwise equal numbers of each   
        else
            nOdd = nHidden/2;
        end
    % - - > > #
    
    % Reorder coefficients
        vHiddenR = zeros(nHidden,1);
        vHiddenR(1:2:end) = vHidden(1:nOdd);
        vHiddenR(2:2:end) = vHidden(nOdd+1:end);
    % - - > > #
end


function vHidden = ReorderCnRand(vHidden,publicKey)

    rng(publicKey);
    
    % Get random permutation
        nHidden = numel(vHidden);

        vPerm = randperm(nHidden);
    % - - > > #
    
    % Apply permutation
        vHidden(vPerm) = vHidden;
    % - - > > #
end

