function [cIndex, mNCnPerBlock] = ExpandIndexGrey(mIndexFolded,nXBlocks,nYBlocks...
    ,nIndexes,vNCnPlain,sBlockDims,sRescale,vSize,publicKey,privateKey,rotate,...
    sTuning)
       
% PrintWithStars([ mfilename ,' Routine Called']);

if nargin < 12
    sTuning.vTuningFactor = 1;
    sTuning.quantMethod = 'none';
    sTuning.scaleEmbeddedImage = 0;
    sTuning.displayAnalysis = 0;
    sTuning.hostMagnitude = 0;
    sTuning.hideIndex = 1;
    sTuning.encodeIndex = 0;
    sTuning.intensityAdjustment = 1;
    sTuning.rescaleIndex = 1;
    sTuning.method = 1;
end

% Show some info
% < < - - #
    showInfo = true;
% - - > > #

% Set up the key structure
    sKey.publicKey = publicKey;
    sKey.privateKey = privateKey;
    sKey.rotate = rotate;
% - - > > #

% Initialize the folded index structure
    sIndexFolded = SIndexFolded;
    [sIndexFolded.y sIndexFolded.x sIndexFolded.z] = size(mIndexFolded);
    sIndexFolded.nIndexes = nIndexes;    
% - - > > #    

% Block the matrix
% < < - - #
    sIndexFolded.cIndexFolded = BlockMImage(mIndexFolded, sBlockDims.indexHeight...
        ,sBlockDims.indexWidth);
    sIndexFolded.blockWidth = sBlockDims.indexWidth;
    sIndexFolded.blockHeight = sBlockDims.indexHeight;
    [sIndexFolded.nYBlocks sIndexFolded.nXBlocks] = ...
        size(sIndexFolded.cIndexFolded);
    sIndexFolded.nPixelsBlock = sIndexFolded.blockWidth * sIndexFolded.blockHeight;
% - - > > #

% Seperate the host and hidden images
    start = clock;
    sHidden = SeparateCellIndex(sIndexFolded, sKey);       
    removeTime = etime(clock,start);
    
    if showInfo
        fprintf('Time to seperate the embedded image (F) from the host image: %.2f seconds\n'...
            ,removeTime);
    end
% - - > > #

% Remove the padding from the hidden indexes
    sHidden.vIndex= sHidden.vIndex(1:sIndexFolded.nIndexes); 
% - - > > #

% Rescale hidden vars back to original values
    sHidden = RescaleSHidden(sHidden, sRescale);   
% - - > > #

    % Test replace vNIndex with random numbers between 0 and 64
%         rng(300);
%         vNCnPlain = ceil(rand(1,numel(vNCnPlain))*64);
    % - - > > #


% Reconstruct vector containing the number of indexes picked per block
    sHidden.vNIndex = [sHidden.vNIndex, vNCnPlain];
% - - > > #

% Place # of indexes per block back in the correct order
%     sHidden.vNIndex = RRandPerm(sHidden.vNIndex,sKey.privateKey);
% - - > > #

% Return matrix containing # of indexes per block
    mNCnPerBlock = zeros(nYBlocks,nXBlocks);
    mNCnPerBlock(:) = sHidden.vNIndex;
    mNCnPerBlock = round(mNCnPerBlock);                                           
        
    % Check it is consistent with nIndexes
    if sum(mNCnPerBlock(:)) ~= sIndexFolded.nIndexes

        fprintf('The number of coefficients per block: %i, is not consistent with the number of coefficients: %i\n',...
            sum(mNCnPerBlock(:)), sIndexFolded.nIndexes);
        fprintf('Using randomly generated number of atoms per block');
        vRand = RandSum(nYBlocks*nXBlocks,sIndexFolded.nIndexes,...
            sIndexFolded.nPixelsBlock-1);
        mNCnPerBlock(:) = vRand;

    end       
% - - > > #

% Place indexes into cIndex adjust nCoefficients if mNCnPerBlock has
% changed
    [cIndex mNCnPerBlock] = PopulateCIndex(sHidden.vIndex,mNCnPerBlock,1,...
        sum(vSize)^2);
% - - > > #

end

%% Begin sub functions
function sHidden = RescaleSHidden(sHidden,sRescale)

   if sRescale.rescaled == 1       
       
       sHidden.vNIndex = round(Rescale(sHidden.vNIndex, sRescale.minV,...
           sRescale.maxV));       
   
   elseif sPlainText.rescaled == 2
             
       sHidden.vIndex = round(Rescale(sHidden.vIndex, sRescale.minV,...
           sRescale.maxV));
       
   end

end

function [cIndex mNCnPerBlockNew] = PopulateCIndex(vCn,mNCnPerBlock,minIndex,...
    maxIndex)
    [y x] = size(mNCnPerBlock);
    mNCnPerBlockNew = zeros(y,x);
    cIndex = cell(y,x);
    iVCn = 1;
    rowIncrement = 1;
    iXBlockStart = 1;
    rangeIndex = maxIndex - minIndex;
    
    for i = 1:y

        for j = iXBlockStart:rowIncrement:x
                
            % Apply rounding if out of range set to nearest value in the correct 
            % range
            
            vCnLoop = vCn(iVCn:iVCn+mNCnPerBlock(i,j)-1);            
            vCnRounded = round(vCnLoop)';
            
%             % Remove out of range indexes and adjust mNCnPerBlock                
%                 iToKeep = (vCnRounded >= minIndex) & (vCnRounded <= maxIndex);
%                 vCnRounded = vCnRounded(iToKeep);
%                 % If no coefficients for this block create a random one
%                     if numel(vCnRounded) == 0
%                         vCnRounded = ceil(rand(1)*rangeIndex)- minIndex+1;
%                     end
%                 % - - > >
%                 mNCnPerBlockNew(i,j) = numel(vCnRounded);
%             % - - > > #            

            % Set out of range values to random numbers in the correct
            % range
                iRand = (vCnRounded < minIndex) | (vCnRounded > maxIndex);
                vCnRounded(iRand) = ceil(rand(1)*rangeIndex)- minIndex+1;
%                 vCnRounded(iRand) = 1;
                
            % - - > > #
            
            cIndex{i,j}.iMDictionary2D = vCnRounded;
            iVCn = iVCn + mNCnPerBlock(i,j);
            
        end
          
    end
    
end

% For DPCM
%{
function cIndex = PopulateCIndex(vCn,mNCnPerBlock,minIndex,maxIndex)
    [y x] = size(mNCnPerBlock);
    cIndex = cell(y,x);
    iVCn = 1;
    rowIncrement = 1;
    iXBlockStart = 1;
    
    for i = 1:y

        for j = iXBlockStart:rowIncrement:x
    
            
            % Apply rounding if out of range set to nearest value in the correct range
            vCnRounded = round(vCn(iVCn:iVCn+mNCnPerBlock(i,j)-1))';
            % Find less than min
            iLessThanMin = vCnRounded < minIndex;
            vCnRounded(iLessThanMin) = minIndex;
            iMoreThanMax = vCnRounded > maxIndex;
            vCnRounded(iMoreThanMax) = maxIndex;
            cIndex{i,j}.iMDictionary2D = vCnRounded;
            iVCn = iVCn + mNCnPerBlock(i,j);
            
        end
        
        temp = iXBlockStart;
        iXBlockStart = x;
        x = temp;
        rowIncrement = rowIncrement * -1;
        
    end
    
end
%}
