function cImage = FlattenR(vImage,blockHeight,vNPixels,nColumns)

% Reverses the steps in Flatten
% Takes a vector of pixles and reconstructs the cell arrays it was taken
% from.

nColours = numel(vNPixels);
cImage = cell(1,nColours);
iVImage = 1;

for iColour = 1:nColours

    % Get the number of rows and row blocks
        nRowBlocks = ceil(vNPixels(iColour)/(nColumns*blockHeight));
        nRows = nRowBlocks*blockHeight;
        nFullRows = (nRowBlocks-1)*blockHeight;  
    % -->>
    
    % Initialize the cell for this image plain
        cImage{iColour} = zeros(nRows,nColumns);
    % - - > > #
    
    % Number of useful columns in the last row 
    %   Any pixels in the last blockHeight rows extending beyond this
    % column contain zeros used as padding.
        nPixelsFull = ((nRowBlocks-1)*blockHeight*nColumns);
        nPixelsRemaining = vNPixels(iColour) - nPixelsFull;
        nColumnsRemaining = (nPixelsRemaining/blockHeight);
    % -->>
    
    % Reshape the vector into an image matrix
        % First take all the full rows
            cImage{iColour}(1:nFullRows,:) = reshape(vImage(iVImage:...
                iVImage+nPixelsFull-1),nFullRows,nColumns);
        % - - > >
        
        % Now take the remaining blocks
            cImage{iColour}(nFullRows+1:end,1:nColumnsRemaining) = reshape(...
                vImage(iVImage+nPixelsFull:iVImage+vNPixels(iColour)-1),blockHeight,...
                nColumnsRemaining);
        % - - > >
    % - - > > #
    
    iVImage = iVImage + vNPixels(iColour);


end
