function [ mFoldedImage mEmbeddedImage sImageHost iXBlockEnd] = ...
    FoldApproxGrey(sImageApprox,sDictionary,sBlockDims,publicKey,privateKey,...
    rotate,sTuning)

% PrintWithStars([mfilename ,' Routine Called']);

if nargin < 3
    sBlockDims.originalHeight = sImageApprox.blockHeight;
    sBlockDims.originalWidth = sImageApprox.blockWidth;
    sBlockDims.foldedHeight = 8;
    sBlockDims.foldedWidth = 8;
end
if nargin < 4
    publicKey = 7;
end
if nargin < 5
    privateKey = 123456789;
end
if nargin < 6
    rotate = 1;
end
if nargin < 7
    sTuning.vTuningFactor = 1;
    sTuning.quantMethod = 'none';
    sTuning.scaleEmbeddedImage = 0;
    sTuning.displayImages = 0;
    sTuning.debug = 0;
    sTuning.hostMagnitude = 0;
    sTuning.hideIndex = 0;
    sTuning.encodeIndex = 0;
    sTuning.intensityAdjustment = 0;
    sTuning.method = 1;
end

% Show some info
% < < - - #
    showInfo = true;
% - - > > #

% Set up the key structure
% < < - - #
    sKey.publicKey = publicKey;
    sKey.privateKey = privateKey;
    sKey.rotate = rotate;
% - - > > #

nPixels = sImageApprox.y*sImageApprox.x;

% Calculate the dimensions of the folded section
%   Calculate the dimensions of the host/folded image as nYBlocksNew X nXBlocks
% where iXBlockEnd is the index of the last block used on row nYBlocksNew.  
% The remaining nXBlocks - iXBlocksEnd are padded with zeros. lFree is the 
% amount of free positions left, unused space in simulation.
% < < - - #...
    
    [nYBlocksNew, iYBlockStart, iXBlockEnd, iXBlockStart, lFree, AdjustedSR]...
        = CalculateNewDimensions(nPixels, sImageApprox.nCoefficients, ...
        sImageApprox.nXBlocks, sImageApprox.blockWidth);
% - - > > #
    
% Set up image host structure
% < < - - #
    sImageHost = SImageHost;
    sImageHost.blockWidth = sBlockDims.approxWidth;
    sImageHost.blockHeight = sBlockDims.approxHeight;
    sImageHost.nXBlocks = sImageApprox.nXBlocks;
    sImageHost.nYBlocks = nYBlocksNew;
    sImageHost.x = sImageApprox.x;
    sImageHost.y = sImageHost.nYBlocks*sImageHost.blockWidth;
    sImageHost.iLastXBlock = iXBlockEnd;
    sImageHost.SR = AdjustedSR;
    sImageHost.iLastX = iXBlockEnd*sImageHost.blockWidth;
    sImageHost.x = sImageApprox.x;
    sImageHost.y = sImageHost.nYBlocks*sImageHost.blockWidth;
% - - > > #

% Get the host image
% < < - - #
    sImageHost.mImageHost = zeros(sImageHost.y,sImageHost.x);
    sImageHost.mImageHost(1:sImageHost.y-sImageHost.blockWidth,:) = ...
        sImageApprox.mImageApprox(1:sImageHost.y-sImageHost.blockWidth,:);
    sImageHost.mImageHost(sImageHost.y-sImageHost.blockWidth + 1:sImageHost.y...
        ,1:sImageHost.iLastX) = sImageApprox.mImageApprox(...
        sImageHost.y-sImageHost.blockWidth + 1:sImageHost.y,...
        1:sImageHost.iLastX);
% - - > > #

% Report the size
% < < - - #
%     if showInfo
%         fprintf('Folded Image: %i x %i pixels, %i x %i blocks\n',sImageHost.y,...
%             sImageHost.x,sImageHost.nYBlocks,sImageHost.nXBlocks);
%     end
% - - > > #

if sTuning.debug
    PrintWithStars('Dimensions');
    fprintf('Original Image: %ix%i pixels, %ix%i blocks\n',sImageApprox.y,sImageApprox.x,sImageApprox.nYBlocks,sImageApprox.nXBlocks);
    fprintf('Folded Image: %ix%i pixels, %ix%i blocks\n',sImageHost.y,sImageHost.x,sImageHost.nYBlocks,sImageHost.nXBlocks);
    fprintf('Row index of last block: %i, Space left: %i\n',iXBlockEnd,lFree);
    fprintf('Adjusted SR: %.4f\n',AdjustedSR);
end

% Get the coefficients to fold into the host image
%   i.e. the coefficients not used to construct the host image
% < < - - #
    vCnHidden = GetCnToFold(sImageApprox.cCoefficients,...
        sImageApprox.cIndex,iXBlockStart,iYBlockStart,sImageApprox.nYBlocks, ...
        sImageApprox.nXBlocks, sImageApprox.nCoefficients);
% - - > > #

% Reorder the coefficients
%     vCnHidden = RandPerm(vCnHidden,sKey.publicKey);
% - - > > #

% Add required padding
%   Required so hidden vector has the same number of elements as free space
% in the host image.
% NOTE: If not using an orthogonal basis to hide the coefficients this space
% would be distributed over all the blocks to avoid the calculation of the
% projector being ill posed, as the proctor will be orthogonal this step is
% unecessary.
% < < - - #
    vCnHidden = [vCnHidden; zeros(lFree,1)];
% - - > > #

% Calculate the embedded image
% < < - - #
    start = clock;
    mDictionary2D = kron(sDictionary.mDictionary,sDictionary.mDictionary);
    mEmbeddedImage = GenerateEmbeddedImage(vCnHidden, mDictionary2D,...
        sImageApprox.cIndex(1:nYBlocksNew,:), iXBlockEnd, sKey);
    elapsedTime = etime(clock,start);
% - - > > #

if showInfo
%     PrintWithStars('Folding');
    fprintf('Folding Image: %.2f seconds\n',elapsedTime);
end

if sTuning.debug
    fprintf('Key to change to prevent plain text attacks %i\n',publicKey);
    fprintf('Key to prevent unorthorized recovery %i\n',privateKey);
    if rotate
        fprintf('Orthogonal basis hidden using random rotation\n');
    else
        fprintf('Orthogonal basis hidden using random permutation\n');
    end
    minEmbeddedImage = min(mEmbeddedImage(:));
    maxEmbeddedImage = max(mEmbeddedImage(:));
    meanEmbeddedImage = mean(mEmbeddedImage(:));
    fprintf('Range of intensities of embedded image\n');
    fprintf('Min: %.0f, Max %.0f, Mean: %.2f\n',minEmbeddedImage,maxEmbeddedImage,meanEmbeddedImage);
end

% Fold the image
% < < - - #
    mFoldedImage = sImageHost.mImageHost + mEmbeddedImage;
% - - > > #

end

% Start of sub functions #######################################################

function [nYBlocks, iYBlockStart, iXBlockEnd, iXBlockStart, lFree, AdjustedSR] = ...
    CalculateNewDimensions(nPixels, nCoefficients, nXBlocks, blockWidth)

    nBlocks = ceil(nCoefficients/blockWidth^2);
    lFree = (nBlocks*blockWidth^2) - nCoefficients;
    nYBlocks = ceil(nBlocks/nXBlocks);
    iXBlockEnd = nXBlocks - (nYBlocks*nXBlocks - nBlocks);
    if iXBlockEnd == nXBlocks
        iXBlockStart = 1;
        iYBlockStart = nYBlocks +1;
    else    
        iXBlockStart = iXBlockEnd  + 1;
        iYBlockStart = nYBlocks;
    end
    AdjustedSR = nPixels/(nCoefficients+lFree);

end

% Alternative scan pattern for DPCM
function [vHiddenCn,vHiddenIndex] = GetCnToFold(cCn,cIndex,...
    iXBlockStart,nYBlocksNew,nYBlocks, nXBlocks, nCoefficients)

    vHiddenCn = zeros(nCoefficients,1);
    vHiddenIndex = zeros(nCoefficients,1);
    iHidden = 1;
    rowIncrement = 1;
    %% Scan in the order we will apply the DPCM
    for i = nYBlocksNew:nYBlocks

        for j = iXBlockStart:rowIncrement:nXBlocks

            nCn = numel(cCn{i,j});        
            vHiddenCn(iHidden:iHidden+nCn-1) = cCn{i,j};
%             vHiddenIndex(iHidden:iHidden+nCn-1) = cIndex{i,j}.vIMDictionary2D;
            vHiddenIndex(iHidden:iHidden+nCn-1) = cIndex{i,j};

            iHidden = iHidden + nCn;

        end
        if i == nYBlocksNew
            iXBlockStart = 1;
        end
        temp = iXBlockStart;
        iXBlockStart = nXBlocks;
        nXBlocks = temp;
        rowIncrement = rowIncrement * -1;
    end
    vHiddenCn = vHiddenCn(1:iHidden-1);  

end




