function [ mFoldedImage,mEmbeddedImage,sImageHost,iXBlockEnd] = FoldApproxMl(...
    sImageApprox,sDictionary,sBlockDims,publicKey,privateKey,rotate,sTuning)

% PrintWithStars([mfilename ,' Routine Called']);

if nargin < 3
    sBlockDims.originalHeight = sImageApprox.blockHeight;
    sBlockDims.originalWidth = sImageApprox.blockWidth;
    sBlockDims.foldedHeight = 8;
    sBlockDims.foldedWidth = 8;
end
if nargin < 4
    publicKey = 7;
end
if nargin < 5
    privateKey = 123456789;
end
if nargin < 6
    rotate = 1;
end
if nargin < 7
    sTuning.vTuningFactor = 1;
    sTuning.quantMethod = 'none';
    sTuning.scaleEmbeddedImage = 0;
    sTuning.displayImages = 0;
    sTuning.debug = 0;
    sTuning.hostMagnitude = 0;
    sTuning.hideIndex = 0;
    sTuning.encodeIndex = 0;
    sTuning.intensityAdjustment = 0;
    sTuning.method = 1;
end

% Show some info
% < < - - #
    showInfo = true;
% - - > > #

% Set up the key structure
% < < - - #
    sKey.publicKey = publicKey;
    sKey.privateKey = privateKey;
    sKey.rotate = rotate;
% - - > > #

% # Per colour plain
%   Each folded plain will have the same dimensions
% < < - - #
    nPixelsPerPlain = sImageApprox.y*sImageApprox.x;
    nCoeffPerPlain = sImageApprox.nCoefficients/sImageApprox.z;
% - - > > #

% Calculate the dimensions of the folded section
%   Calculate the dimensions of the host/folded image as nYBlocksNew X nXBlocks
% where iXBlockEnd is the index of the last block used on row nYBlocksNew.  
% The remaining nXBlocks - iXBlocksEnd are padded with zeros. lFree is the 
% amount of free positions left, unused space in simulation.
% < < - - #
    [nYBlocksNew, iYBlockStart, iXBlockEnd, iXBlockStart, lFree, AdjustedSR]...
        = CalculateNewDimensions(nPixelsPerPlain, nCoeffPerPlain, ...
        sImageApprox.nXBlocks, sImageApprox.blockWidth);
% - - > > #
    
% Set up image host structure
% < < - - #
    sImageHost = SImageHost;
    sImageHost.blockWidth = sBlockDims.approxWidth;
    sImageHost.blockHeight = sBlockDims.approxHeight;
    sImageHost.nXBlocks = sImageApprox.nXBlocks;
    sImageHost.nYBlocks = nYBlocksNew;
    sImageHost.x = sImageApprox.x;
    sImageHost.y = sImageHost.nYBlocks*sImageHost.blockWidth;
    sImageHost.z = sImageApprox.z;
    sImageHost.iLastXBlock = iXBlockEnd;
    sImageHost.SR = AdjustedSR;
    sImageHost.iLastX = iXBlockEnd*sImageHost.blockWidth;
    sImageHost.x = sImageApprox.x;
    sImageHost.y = sImageHost.nYBlocks*sImageHost.blockWidth;
% - - > > #

% Get the host image
% < < - - #
    sImageHost.mImageHost = zeros(sImageHost.y,sImageHost.x,sImageHost.z);
    sImageHost.mImageHost(1:sImageHost.y-sImageHost.blockWidth,:,:) = ...
        sImageApprox.mImageApprox(1:sImageHost.y-sImageHost.blockWidth,:,:);
    sImageHost.mImageHost(sImageHost.y-sImageHost.blockWidth + 1:sImageHost.y...
        ,1:sImageHost.iLastX,:) = sImageApprox.mImageApprox(...
        sImageHost.y-sImageHost.blockWidth + 1:sImageHost.y,...
        1:sImageHost.iLastX,:);
% - - > > #

% Report the size
% < < - - #
    if showInfo
        fprintf('Folded Image: %i x %i x %i pixels, %i x %i blocks\n',sImageHost.y,...
            sImageHost.x,sImageHost.z,sImageHost.nYBlocks,sImageHost.nXBlocks);
    end
% - - > > #

if sTuning.debug
    PrintWithStars('Dimensions');
    fprintf('Original Image: %ix%i pixels, %ix%i blocks\n',sImageApprox.y,sImageApprox.x,sImageApprox.nYBlocks,sImageApprox.nXBlocks);
    fprintf('Folded Image: %ix%i pixels, %ix%i blocks\n',sImageHost.y,sImageHost.x,sImageHost.nYBlocks,sImageHost.nXBlocks);
    fprintf('Row index of last block: %i, Space left: %i\n',iXBlockEnd,lFree);
    fprintf('Adjusted SR: %.4f\n',AdjustedSR);
end

% Repeat creation of embedded image for each colour plain
% < < - - #
% Initialize embeded image
% < < - - #
    mEmbeddedImage = zeros(sImageHost.y,sImageHost.x,sImageHost.z);
% - - > > #

for iColour = 1:sImageHost.z        
    % Get the coefficients to fold into the host image
    %   i.e. the coefficients not used to construct the host image
    % < < - - 
        vCnHidden = GetCnToFold(sImageApprox.cCoefficients,...
            sImageApprox.cIndex,iXBlockStart,iYBlockStart,...
            sImageApprox.nYBlocks, sImageApprox.nXBlocks,...
            nCoeffPerPlain,iColour);
    % - - > > 

    % Reorder the coefficients
%         vCnHidden = RandPerm(vCnHidden,sKey.publicKey);
    % - - > > #    
        
    % Add required padding
    %   Required so hidden vector has the same number of elements as free space
    % in the host image.
    % NOTE: If not using an orthogonal basis to hide the coefficients this space
    % would be distributed over all the blocks to avoid the calculation of the
    % projector being ill posed, as the proctor will be orthogonal this step is
    % unecessary.
    % < < - - 
        vCnHidden = [vCnHidden; zeros(lFree,1)];
    % - - > > 
    
    % Calculate the embedded image
    % < < - - 
        start = clock;
        mDictionary2D = kron(sDictionary.mDictionary,sDictionary.mDictionary);
        mEmbeddedImage(:,:,iColour) = GenerateEmbeddedImage(vCnHidden,...
            mDictionary2D,sImageApprox.cIndex(1:nYBlocksNew,:),iXBlockEnd,sKey);
        elapsedTime = etime(clock,start);
    % - - > > 
    
    % Increment the public key
        sKey.publicKey = sKey.publicKey + 1;
    % - - > >

    if showInfo
%         PrintWithStars('Folding');
        fprintf('Folding plain %i:- %.2f seconds\n',iColour,elapsedTime);
    end

    if sTuning.debug
        fprintf('Key to change to prevent plain text attacks %i\n',publicKey);
        fprintf('Key to prevent unorthorized recovery %i\n',privateKey);
        if rotate
            fprintf('Orthogonal basis hidden using random rotation\n');
        else
            fprintf('Orthogonal basis hidden using random permutation\n');
        end
        minEmbeddedImage = min(mEmbeddedImage(:));
        maxEmbeddedImage = max(mEmbeddedImage(:));
        meanEmbeddedImage = mean(mEmbeddedImage(:));
        fprintf('Range of intensities of embedded image\n');
        fprintf('Min: %.0f, Max %.0f, Mean: %.2f\n',minEmbeddedImage,...
            maxEmbeddedImage,meanEmbeddedImage);
    end
end
% - - > > #
    
% Fold the image
% < < - - #
    mFoldedImage = sImageHost.mImageHost + mEmbeddedImage;
% - - > > #

% Scramble the folded blocks
%     % Block the image
%     [cImage mImageResized] = BlockMImage(mFoldedImage,sImageApprox.blockWidth,...
%         sImageApprox.blockWidth);
%     % Remove the plain text
%     nBlocks = (nYBlocksNew-1)*sImageHost.nXBlocks + iXBlockEnd;
%     cImage = cImage';
%     cImageNew = cImage(1:nBlocks);
%     % Perm the blocks
%     randPerm = randperm(nBlocks);
%     cImageNew = cImageNew(randPerm);
%     % Place back into cImage
%     cImage(1:nBlocks) = cImageNew;
%     cImage = cImage';
%     % Place back in a matrix
%     mFoldedImage = cell2mat(cImage);
% - - > >
   
end


% Start of sub functions #######################################################

function [nYBlocks, iYBlockStart, iXBlockEnd, iXBlockStart, lFree, AdjustedSR] = ...
    CalculateNewDimensions(nPixels, nCoefficients, nXBlocks, blockWidth)

    nBlocks = ceil(nCoefficients/blockWidth^2);
    lFree = (nBlocks*blockWidth^2) - nCoefficients;
    nYBlocks = ceil(nBlocks/nXBlocks);
    iXBlockEnd = nXBlocks - (nYBlocks*nXBlocks - nBlocks);
    if iXBlockEnd == nXBlocks
        iXBlockStart = 1;
        iYBlockStart = nYBlocks +1;
    else    
        iXBlockStart = iXBlockEnd  + 1;
        iYBlockStart = nYBlocks;
    end
    AdjustedSR = nPixels/(nCoefficients+lFree);

end

%{
function sCn = GetAllIndexes(cIndex,nCoefficients)


    %sCn.vCn = zeros(sImage.nCoefficients,1);
    sCn.vICn = zeros(nCoefficients,1);
    [nRows nColumns] = size(cIndex);
    sCn.mNCnPerBlock = zeros(nRows,nColumns);

    iHidden = 1;
    rowIncrement = 1;
    iXBlockStart = 1;
    % Needed as we change the scan order for each row
    nXBlocksLoop = nColumns;
    %% Scan in the order we will apply the DPCM
    for i = 1:nRows

        for j = iXBlockStart:rowIncrement:nXBlocksLoop

            nCn = numel(cIndex{i,j}.iMDictionary2D);        
            sCn.vCn(iHidden:iHidden+nCn-1) = sImage.cCoefficients{i,j};
            sCn.vICn(iHidden:iHidden+nCn-1) = cIndex{i,j}.iMDictionary2D;
            sCn.mNCnPerBlock(i,j) = nCn;
            iHidden = iHidden + nCn;

        end
        temp = iXBlockStart;
        iXBlockStart = nXBlocksLoop;
        nXBlocksLoop = temp;
        rowIncrement = rowIncrement * -1;
    end
    sCn.vCn = sCn.vCn(1:iHidden-1);
    sCn.vICn = sCn.vICn(1:iHidden-1);    
    


end
%}

function [vHiddenCn,vHiddenIndex] = GetCnToFold(cCn,cIndex,...
    iXBlockStart,nYBlocksNew,nYBlocks, nXBlocks, nCoefficients,iColour)

    vHiddenCn = zeros(nCoefficients,1);
    vHiddenIndex = zeros(nCoefficients,1);
    iHidden = 1;
    rowIncrement = 1;
    %% Scan in the order we will apply the DPCM
    for i = nYBlocksNew:nYBlocks

        for j = iXBlockStart:rowIncrement:nXBlocks

            nCn = size(cCn{i,j},2);        
            vHiddenCn(iHidden:iHidden+nCn-1) = cCn{i,j}(iColour,:);
            vHiddenIndex(iHidden:iHidden+nCn-1) = cIndex{i,j};
            iHidden = iHidden + nCn;

        end
        if i == nYBlocksNew
            iXBlockStart = 1;
        end
        temp = iXBlockStart;
        iXBlockStart = nXBlocks;
        nXBlocks = temp;
        rowIncrement = rowIncrement * -1;
    end
    vHiddenCn = vHiddenCn(1:iHidden-1);  

end

%{
function vQuantIndex = GetQuantIndex(approxMethod,blockWidth)

    % In future will be adaptive based on coefficient values.
    if strcmp(approxMethod, 'DCT')

            vQuantIndex = 1;

    elseif strcmp(approxMethod(1:6), 'RDCTDB') 

        iD1 = [1;1;1;1];
        iV1 = [1;1;2;2];
        iD2 = [1;1;1;1];
        iV2 = [1;2;1;2];
        vDictSize = [(2*blockWidth-1); blockWidth];
        vQuantIndex = KroneckerIndex(iD1,iD2,iV1,iV2,vDictSize);

    end
    
end
%}

%{
function sHidden = RescaleIndex(sHidden)

    % Rescales the coefficients, the indexes, the number of atoms chosen
    % per block and the number of x and y blocks to have the same range
    % of values.
    % Use the maximum of the three ranges
    % The parameters will be stored as
    % [coefficients, indexes, extra]
    % Therefore recaled = 1 means the coefficients have been rescaled and
    % vMax1 refers to the indexes and vMax2 to the extra params etc.
    
    % Were the coefficients rescaled
    % <<--
    % Default to zero to check for errors
    sHidden.rescaled = 0;
    % -->>
    
    
    % Max, min and range of the coefficients
    % <<--
    minCn = min(sHidden.vCn);
    maxCn = max(sHidden.vCn);
    rangeCn = maxCn-minCn;
    % -->>
    
    % Max, min and range of the coefficients
    % <<--
    minICn = min(sHidden.vICn);
    maxICn = max(sHidden.vICn);
    rangeICn = maxICn-minICn;
    % -->>      
    
    % Max, min and range of the number of atoms required for each block and
    % the number or y and x blocks
    % <<--
    minExtra = min(sHidden.vExtra);
    maxExtra = max(sHidden.vExtra);
    rangeExtra = maxExtra-minExtra;
    % -->>     
    
    %{
    % Rescale using integer rescaling params
    % Inefficient method, easy to follow
    % <<--
    % If the range used by the coefficients is greatest
    if rangeICn > rangeExtra
        
        % Rescale the extra params
        [sHidden.vExtra sHidden.minV sHidden.maxV] = RescaleInt(sHidden.vExtra,minICn,maxICn);
        
        % Needed for recovery
        sHidden.rescaled = 1;
        
    else
           
        % Rescale the indexes
        [sHidden.vICn sHidden.minV sHidden.maxV] = RescaleInt(sHidden.vICn,minExtra,maxExtra);
        
        % Needed for recovery
        sHidden.rescaled = 2;
        
    end
    %}
    
    
    if (rangeCn > rangeICn) && (rangeCn > rangeExtra)
        
        % Rescale the indexes
        [sHidden.vICn sHidden.minV1 sHidden.maxV1] = Rescale(sHidden.vICn,minCn,maxCn);
        
        % Rescale the extra params
        [sHidden.vExtra sHidden.minV2 sHidden.maxV2] = Rescale(sHidden.vExtra,minCn,maxCn);
        
        % Needed for recovery
        sHidden.rescaled = 1;
        
    % If the range used by indexes is greatest
    elseif (rangeICn > rangeCn) && (rangeICn > rangeExtra)
        
        % Rescale the coefficients
        [sHidden.vCn sHidden.minV1 sHidden.maxV1] = Rescale(sHidden.vCn,minICn,maxICn);
        
        % Rescale the extra params
        [sHidden.vExtra sHidden.minV2 sHidden.maxV2] = Rescale(sHidden.vExtra,minICn,maxICn);
        
        % Needed for recovery
        sHidden.rescaled = 2;
        
    % If the range used by the extra params is greatest
    else
        
        % Rescale the coefficients
        [sHidden.vCn sHidden.minV1 sHidden.maxV1] = Rescale(sHidden.vCn,minExtra,maxExtra);
        
        % Rescale the extra params
        [sHidden.vICn sHidden.minV2 sHidden.maxV2] = Rescale(sHidden.vICn,minExtra,maxExtra);
        
        % Needed for recovery
        sHidden.rescaled = 3;        
        
    end
    
    % -->>
        

end
%}