function mEncryptionOperator = GenEncryptOp(mAtoms,x,mRandPrivate,rotate,...
    mPOrthSpace)
% GenerateEncryptionOperator Generates encryption operator for the FoldImage and ExpandImage
% routines.
%
% First the routine calculates the orthogonal complement of the approximated image space
% using a random matrix generated with a public key (c.f. eq. 11 and 12 in [1]) to avoid
% plain text attacks.
% Then it rotates (or permutes) these vectors using a a random matrix generated with a
% private key to obtain the encryption operator (c.f. eq. 13 in [1]).
%
% Usage:    mEncryptionOperator = GenerateEncryptionOperator( mAtoms, x, sKey, mPOrthSpace );
%
% Inputs:
%   mAtoms          Atoms spanning the approximated image space.
%   x               Number of used to construct the encryption operator.
%   sKey            Structure containing encryption variables.
%   mPOrthSpace     Orthogonal projection matrix onto the mAtoms.
%
% Outputs:
%   mEncryptionOperator     Encryption operator.

% mAtoms = rand(size(mAtoms));

if nargin < 6
    [mAtomsOrth  mRTemp]= qr(mAtoms,0);
    mPOrthSpace = mAtomsOrth*mAtomsOrth';
end

% % Switch to public stream
%     RandStream.setGlobalStream(streamPublic);
% % - - > > #
nRows = size(mAtoms,1);
mRandPublic = rand(nRows,x);
mTemp = mRandPublic - mPOrthSpace*mRandPublic;

% Quickly normalize
    %mTemp = sqrt(sum(mTemp.^2))

% mOrthogonalAtoms = mTemp;

% Check
% [a mT] = qr(mRandPublic*mRotate,0);
% [b mT] = qr(mPOrthSpace*mRandPublic*mRotate,0);
% c = a - b;




% mOrthogonalAtoms = mRandPublic;
% mOrthogonalAtoms = orth(mRandPublic - mPOrthSpace*mRandPublic);
% s = RandStream.create('mt19937ar','seed',sKey.privateKey);
% RandStream.setDefaultStream(s);
% rng(sKey.privateKey);
% rng(987654321);

if rotate
    mRTrans = mRandPrivate(1:x,1:x);
%     mEncryptionOperator = mOrthogonalAtoms*mRotate;
    
    [mEncryptionOperator mRTemp] = qr(mTemp*mRTrans,0);
    
%  mTest = mOrthogonalAtoms*mRotate;
%     mEncryptionOperator = mOrthogonalAtoms;
    
else
    % Obsolete
%     vRandPerm = randperm(x);
%     mEncryptionOperator = mOrthogonalAtoms(:,vRandPerm);
    
end
