function sImageEmbedded = GenerateEmbeddedImageOneAtom(sImageHost, sHidden,...
    sKey)
% GenerateEmbeddedImage Used by FoldImage to embed coefficients in a generated image (c.f. 
% eq. 8 in [1]).
% As we are only using one atom we increment the public key by one for every block.

% Get size of hidden image
%[nYBlocks,nXBlocks] = size(cIndex);
% Inherit from sImageHost
sImageEmbedded = sImageHost;
sImageEmbedded = rmfield(sImageEmbedded,'mImageHost');

% Allocate memory for variables
cEmbeddedImage = cell(sImageHost.nYBlocks,sImageHost.nXBlocks);
%blockWidth = sqrt(size(mDictionary,1));
nCnToHide = sImageHost.blockHeight*sImageHost.blockWidth - 1;

% Use constant normalized atom for the image
vAtom = ones(sImageEmbedded.blockHeight*sImageEmbedded.blockWidth,1);
vAtom = vAtom/norm(vAtom);

% Moved to reduce execution time    
%     mOrthSpace = GenerateEncryptionOperator(vAtom,nCnToHide,sKey);
% - - > > #

% % Set up random streams
%     % Public Stream
%         streamPublic = RandStream('mt19937ar','Seed',sKey.publicKey);
%     % - - > >
%     % Private Stream
%         streamPrivate = RandStream('mt19937ar','Seed',sKey.privateKey);
%     % - - > 
% % - - > > #

% Set up random matrices
%   Performed here to decrease processing time.
%   The public matrix should change for each block.
    % Private key - fixed matrix initialized to the largest possible size.
        RngW(sKey.privateKey);
        mRandPrivate =  rand(nCnToHide,nCnToHide) - 0.5;      
    % - - > >
    
    % Public key - initialize seed here
        RngW(sKey.publicKey);
    % - - > >
% - - > >
%% Process first nYBlocks Rows
iVCn = 1;
iVNCn = 1;
for i = 1:sImageEmbedded.nYBlocks - 1
    
    for j = 1:sImageEmbedded.nXBlocks       
        
        mOrthSpace= GenEncryptOp(vAtom,nCnToHide,mRandPrivate,sKey.rotate);
        % Increment public key by one to prevent plain text attacks
%         sKey.publicKey = sKey.publicKey + 1;
        
        % Get the coefficients to hide
        vCnToHide = sHidden.vICn(iVCn:iVCn+nCnToHide-1);
        iVCn = iVCn + nCnToHide;
        
        % Create a new block in the orthogonal space
        vHiddenBlock = mOrthSpace*vCnToHide;
       
        % Add vAtom multiplied by an element of vNCn
        % <<--
            vHiddenBlock = vHiddenBlock + (vAtom * sHidden.vNCn(iVNCn));
            % Increment the index in vNCn
            iVNCn = iVNCn + 1;
        % -->>
        
        cEmbeddedImage{i,j} = zeros(sImageEmbedded.blockHeight,...
            sImageEmbedded.blockWidth);
        cEmbeddedImage{i,j}(:) = vHiddenBlock;
        
    end
    
end
%% Deal with the last row seperatly as their maybe empty blocks
for j = 1:sImageEmbedded.iLastXBlock
    
        % Calculate the orthogonal space used to hide the coefficients
%         iMDictionary = cIndex{nYBlocks,j}.iMDictionary2D;
%         nCn = size(iMDictionary,2);
%         nCnToHide = nPixelsBlock - nCn;
%         mImageSpace = mDictionary(:,iMDictionary);
%         mOrthSpace = GenerateEncryptionOperator(vAtom,nCnToHide,sKey);
                     
        % Increment public key by one to prevent plain text attacks
%         sKey.publicKey = sKey.publicKey + 1;
        mOrthSpace= GenEncryptOp(vAtom,nCnToHide,mRandPrivate,sKey.rotate);
        % Get the coefficients to hide
        vCnToHide = sHidden.vICn(iVCn:iVCn+nCnToHide-1);
        iVCn = iVCn + nCnToHide;
        
        % Create a new block in the orthogonal space
        vHiddenBlock = mOrthSpace*vCnToHide;
        
        % Add vAtom multiplied by an element of vNCn
        % <<--
            vHiddenBlock = vHiddenBlock + (vAtom * sHidden.vNCn(iVNCn));
            % Increment the index in vNCn
            iVNCn = iVNCn + 1;
        % -->>
        
        cEmbeddedImage{sImageEmbedded.nYBlocks,j} = zeros(...
            sImageEmbedded.blockHeight,sImageEmbedded.blockWidth);
        cEmbeddedImage{sImageEmbedded.nYBlocks,j}(:) = vHiddenBlock;
    
end
%% Pad any remaining blocks with zeros
for j = sImageEmbedded.iLastXBlock+1:sImageEmbedded.nXBlocks
    
    cEmbeddedImage{sImageEmbedded.nYBlocks,j} = zeros(...
        sImageEmbedded.blockHeight,sImageEmbedded.blockWidth);
    
end
%% Convert cell image back to matrix
sImageEmbedded.mImageEmbedded = cell2mat(cEmbeddedImage);
