function [mImageApprox mImageResized cCoefficients cIndex nCoefficients ...
    nZeroBlocks] = ...
    MImageApproximation(mImage,error,approach,algorithm,blockWidth,...
    blockHeight,sDictionary,intensityAdjustment)

PrintWithStars([ mfilename, ' routine called']);

if nargin < 4
    error('%s: Not enough input arguments!',functionName);
elseif nargin < 5
    
    blockHeight = 8;
    blockWidth = 8;
    
elseif nargin < 6
    
    blockWidth = 8;
    
end

if nargin < 7
    if ~strcmp(approach,'thresholding')        
        error('%s: Not enough input arguments for approach %s!',mfilename,...
            approach);
    end
end

if nargin < 8
    intensityAdjustment = 0;
end

% Force mImage to be double
% < < - - #
    mImage = double(mImage);
    % Calculate before intensity adjustment
    maxIntensity = max(mImage(:)); % Should be set correctly based on bits
% - - > > #

% Subtract the mean
% < < - - #
    if intensityAdjustment    
        meanPixel = mean(mImage(:));
        mImage = mImage - meanPixel;       
    end
% - - > > #


% If thresholding with wavelets or
if strcmp(approach,'thresholding')
    % REMOVED WAVELET APPROX
    %{
    % If wavelet approximation no need to block the image
    
    if strcmp(algorithm,'wavelets')
        sImageApprox = WaveletApprox(sImageGrey, error);
    %}
    if strcmp(algorithm,'DCT')
        % NEEDS REWRITING FOR RGB IMAGES
        % sImageApprox = ProcessCellImageDCT(sImageBlocked,error);
    end    

elseif strcmp(approach,'greedy2D')     
    
    % approxCriria should be a globel PSNR so get the blockError.
    % Needed for calculation of tolerance below
    % < < - - #
%         nColours = size(mImage,3);
        nPixelsBlock = blockWidth*blockHeight;
    % - - > > #
    
    blockError = PSNRToBlockErrorNorm( error, nPixelsBlock,maxIntensity);
    
    % REDUNDANT
    %{
    if strcmp(algorithm,'RDCTDB') % OMP 2D with MATLAB
        
        sImageApprox = ProcessCellImageRDCTDB(sImageBlocked, blockError); 
        
    else
    %}
    [mImageApprox mImageResized cCoefficients cIndex nCoefficients ...
        nZeroBlocks] = ApproxMImageGreedySep(mImage,blockWidth,blockHeight,...
        blockError,sDictionary,...
        algorithm);

else
    
    error('%s: Approximation method not found!',functionName);
    
end

if intensityAdjustment
    mImageResized = mImageResized + meanPixel;
    mImageApprox = mImageApprox + meanPixel;
end

nPixels = numel(mImageResized);
SR = nPixels/nCoefficients;
maxIntensity = (2^ApproxBitdepth(mImageResized))-1;
PSNR = CalcPSNR(mImageResized,mImageApprox,maxIntensity);
% REMOVED
%{
window = fspecial('gaussian', 11, 1.5);	
K(1) = 0.01;					
K(2) = 0.03;					
sImageApprox.SSIM = ssim(sImageApprox.mImageGrey,...
    sImageApprox.mImageApprox, K,window,sImageApprox.maxIntensity);
if strcmp(algorithm,'wavelets')
    fprintf('Processing %s using %s\n',imagePath,algorithm);
else
%}
fprintf('Processing supplied image matrix using %s in blocks of %i x %i\n',...
    algorithm,blockHeight,blockWidth);
fprintf('The image has an approximate maximum intensity of %i\n',...
    max(mImageResized(:)));
fprintf('SR of %g, achieved for PSNR of %.2f\n',SR,PSNR);