function c = Mat2Cell(m,nRows,nColumns)

% Converts a matrix to fixed sized cells with leading dimensions
% nRowsxnColumns.

% [nRowsM,nColumnsM,nPlains] = size(m);
nRowsM = size(m,1);
nColumnsM = size(m,2);

% Number of blocks
    nRowBlocks = nRowsM/nRows;
    nColumnBlocks = nColumnsM/nColumns;
    
% Validate
    if (nRowBlocks-ceil(nRowBlocks)) || (nColumnBlocks - ceil(nColumnBlocks))
        error('Matrix cannot be exactly blocked!');
    end

% Initialize cell
    c = cell(nRowBlocks,nColumnBlocks);

% Loop and assign arrays to cells
    startColumn = 1;
    endColumn = nColumns;
    for i = 1:nColumnBlocks

        startRow = 1;
        endRow = nRows;
        for j = 1:nRowBlocks

            c{j,i} = m(startRow:endRow,startColumn:endColumn,:);
            startRow = endRow + 1;
            endRow = endRow + nRows;

        end

        startColumn = endColumn + 1;
        endColumn = endColumn + nColumns;

    end      