function vPlainText = PlainTextToVec(sPlainText,cVNCnPlain,maxIntensityToSave)


% Loops through sPlainText placing each value into vPlainText.
%   Each val is reduced to sign,mult and add to ensure less than
% maxIntensityToSave.
%   Loops through any sub structrues, i.e. structs or vectors in sPlainText.
%   The last element ov vPlainText is the number of plain text elements,
% this will let the expander know which image structure to use.
%   Adds the contents of sVNCnPlain to the end.

% Initialize vPlainText large enough for RGB image
    vPlainText = zeros(1,100);
    
    
cFieldNames = fieldnames(sPlainText);
nFields = numel(cFieldNames);

% Keep track of position in vPlainText
%   Post increment, init to 1
% <--
    iVPlainText = 1;
% -->


% try
for iField = 1:nFields
    
    % Stucture of some kind
    sTemp = sPlainText.(cFieldNames{iField});
    
    % If struct process each field
    if isstruct(sTemp)
        
        cFieldsLoop = fieldnames(sTemp);
        nFieldsLoop = numel(cFieldsLoop);
        
        for iFieldLoop = 1:nFieldsLoop
            
            sTempInner = sTemp.(cFieldsLoop{iFieldLoop});
            [mult diff] = Reduce(abs(sTempInner)...
                ,maxIntensityToSave);
            vPlainText(iVPlainText) = GetSign(sTempInner);
            vPlainText(iVPlainText+1) = mult;
            vPlainText(iVPlainText+2) = diff;
            
            iVPlainText = iVPlainText + 3;
            
        end
        
    % If its a cell, each should be a struct
    elseif iscell(sTemp)
        
        if ~isstruct(sTemp{1})
            error('Expecting a cell of structures');
        end
        
        for iCell = 1:numel(sTemp)
            
            sTempInner = sTemp{iCell};
            cFieldsInner = fieldnames(sTempInner);
            nFieldsInner = numel(cFieldsInner);

            for iFieldInner = 1:nFieldsInner

                sTempInnerI = sTempInner.(cFieldsInner{iFieldInner});
                [mult diff] = Reduce(abs(sTempInnerI)...
                    ,maxIntensityToSave);
                vPlainText(iVPlainText) = GetSign(sTempInnerI);
                vPlainText(iVPlainText+1) = mult;
                vPlainText(iVPlainText+2) = diff;

                iVPlainText = iVPlainText + 3;

            end        
        end
        
    else
        
        % Assume a vector or a scalar
        nElements = numel(sTemp);
        
        for iElement = 1:nElements
            
            sTempInner = sTemp(iElement);
            [mult diff] = Reduce(abs(sTempInner),maxIntensityToSave);
            vPlainText(iVPlainText) = GetSign(sTempInner);
            vPlainText(iVPlainText+1) = mult;
            vPlainText(iVPlainText+2) = diff;
            iVPlainText = iVPlainText + 3;
        end
        
    end
    
end

% Reduce the size of vPlainText
    vPlainText = vPlainText(1:iVPlainText-1);
% - - > > #

% Add the vectors in cNCnPlain to the end of vPlainText
% <<--
    for i = 1:numel(cVNCnPlain)
        vPlainText = [vPlainText,cVNCnPlain{i}'];
    end
% -->>

end    

            
function [mult diff] = Reduce(n,sz)

    % Reduct n to multiples of size plus a difference so n can be recovered
    % as n = mult*size + diff;
    
    mult = fix(n/sz);
    diff = n - mult*sz;

end

function signFlag = GetSign(n)

    % Returns a 1 if positive and a 0 if negative
    signFlag = (n >= 0);

end


            
            