function [ sImageRecon,  vCn ] = SeparateCellImage(sImageFolded,...
    sImageRecon,mDictionary,iXBlockEnd,sKey)
% SeparateCellImage Used by ExpandImage to 1) separate the host and embedded image using 
% SplitOrthComp and 2) retrieve the embedded coefficients using GenerateEncryptionOperator.

% % Set up random streams
%     % Public Stream
%         streamPublic = RandStream('mt19937ar','Seed',sKey.publicKey);
%     % - - > >
%     % Private Stream
%         streamPrivate = RandStream('mt19937ar','Seed',sKey.privateKey);
%     % - - > 
% % - - > > #

% Set up random matrices
%   Performed here to decrease processing time.
%   The public matrix should change for each block.
    % Private key - fixed matrix initialized to the largest possible size.
        RngW(sKey.privateKey);
        nRows = size(mDictionary,1);
        mRandPrivate =  rand(nRows-1,nRows -1) - 0.5;      
    % - - > >
    
    % Public key - initialize seed here
        RngW(sKey.publicKey);
    % - - > >
% - - > >

% Initialize the recovered image
    sImageRecon.cImage = cell(sImageRecon.nYBlocks,sImageRecon.nXBlocks);
% - - > > #

    % Allocate memory for variables
%     vCn = zeros(sImageRecon.nCoefficients,1);
    vCn = zeros(sImageRecon.nCoefficients*2,1); %% nPixels
    %cRecoveredImage = cell(size(cIndex));
    cNoise = cell(sImageFolded.nYBlocks,sImageFolded.nXBlocks);
    %% Process first nYBlocks - 1 Rows
    iVCn = 1;
    for i = 1:sImageFolded.nYBlocks - 1

        for j = 1:sImageFolded.nXBlocks        

            % Remove zero indexes
                
            
            % Get the index's of the space to project onto (image space)
            iMDictionary = sImageRecon.cIndex{i,j}.iMDictionary2D;

            % Remove the noise
            [ sImageRecon.cImage{i,j}, vNoise, mPOrthSpace] = SplitOrthComp(...
                sImageFolded.cImage{i,j}, mDictionary(:,iMDictionary));        
            cNoise{i,j} = zeros(sImageRecon.blockWidth);
            cNoise{i,j}(:) = vNoise;

            % Calculate orthogonal space used to recover hidden coefficients
            nCnHidden = sImageRecon.nPixelsBlock - size(iMDictionary,2);
%             mOrthSpace = GenerateEncryptionOperator(mDictionary(:,iMDictionary),...
%                 nCnHidden,sKey, mPOrthSpace);
%             mOrthSpace = GenEncryptOp(mDictionary(:,iMDictionary),nCnHidden...
%                 ,mRandPublic,sKey.rotate,mPOrthSpace);   

            mOrthSpace= GenEncryptOp(mDictionary(:,iMDictionary),nCnHidden,...
                mRandPrivate,sKey.rotate,mPOrthSpace);

            % Recover image coefficients from the noise
            vCn(iVCn:iVCn+nCnHidden-1) = mOrthSpace'*vNoise;
            iVCn = iVCn + nCnHidden;

        end

    end
    %% Deal with the last row seperatly as their maybe empty blocks
    for j = 1:iXBlockEnd

        % Get the index's of the space to project onto (image space)
        iMDictionary = sImageRecon.cIndex{sImageFolded.nYBlocks,j}.iMDictionary2D;

        % Remove the noise
        [ sImageRecon.cImage{sImageFolded.nYBlocks,j}, vNoise, mPOrthSpace] = ...
            SplitOrthComp(sImageFolded.cImage{sImageFolded.nYBlocks,j},...
            mDictionary(:,iMDictionary)); 
        cNoise{sImageFolded.nYBlocks,j} = zeros(sImageRecon.blockWidth);
        cNoise{sImageFolded.nYBlocks,j}(:) = vNoise;

        % Calculate orthogonal space used to recover hidden coefficients
        nCnHidden = sImageRecon.nPixelsBlock - size(iMDictionary,2);
%         mOrthSpace = GenerateEncryptionOperator(mDictionary(:,iMDictionary),...
%             nCnHidden,sKey, mPOrthSpace);
%         mOrthSpace = GenEncryptOp(mDictionary(:,iMDictionary),nCnHidden...
%                 ,mRandPublic,sKey.rotate,mPOrthSpace);
        mOrthSpace= GenEncryptOp(mDictionary(:,iMDictionary),nCnHidden,...
                mRandPrivate,sKey.rotate,mPOrthSpace);

        % Recover image coefficients from the noise
        vCn(iVCn:iVCn+nCnHidden-1) = mOrthSpace'*vNoise;
        iVCn = iVCn + nCnHidden;

    end
    %% Reduce the size of vCn so it only contains coefficients and padding
    vCn = vCn(1:iVCn-1);
    %% Return the noise as a matrix
    for i = iXBlockEnd+1:sImageFolded.nXBlocks
        cNoise{sImageFolded.nYBlocks,i} = zeros(sImageRecon.blockWidth);
    end

end

%% Begin sub functions
function [ mOrth, vOrthComp, mPOrthSpace] = SplitOrthComp(mBlock, mImageSpace)

    %% Remove the noise with orthogonal projection
    [mOrthAtoms mRTemp] = qr(mImageSpace,0);
    mPOrthSpace = mOrthAtoms*mOrthAtoms';
    vBlock = mBlock(:);
    vOrth = (mPOrthSpace*vBlock);
    mOrth = zeros(size(mBlock));
    mOrth(:) = vOrth;
    %% Retrieve the noise
    vOrthComp = vBlock - vOrth;
    
end


