function ShowFig2(mImage)

    % Displays figure 2 for the paper ...
    % mImage should be of the correct uint format
    
    % Determin the number of colour channels
    %   Number of channels in the image + 1 to display the original
    % <<--
        [x y nChannels] = size(mImage);
        clear x y;
    % -->>
    
    
    % If image is not RGB then throw error
    % <<--
%         if nChannels ~= 3
%             error('mImage should be RGB');
%         end
    % -->>
    
    % Establish number of subplots
    %   RGB images will have 4 and grey level images will have 1.
    % <<--
        if nChannels == 3
            
            nSubPlots = 4;
            
        elseif nChannels == 1
            
            nSubPlots = 1;
            
        else
            
            error('mImage should be RGB or grey level');
            
        end
    % -->>
          
    
    % Create figure window
    %   Split into 1 x nChannels + 1 subplots and select the first one 
    % for display.
    % <<--
        figure;
        subplot(1,nSubPlots,1);
    % -->>
    
    % Get the screen dimensions
    % <<--
        vScreen = get(0,'screensize');
        yPixels = vScreen(4);
        xPixels = vScreen(3);
    % -->>

    % Calc figure dimensions
    %   1/2 the size of the display with minimum of 430 x 550
    % <<--        
        yPixelsNew = yPixels/2;
        xPixelsNew = xPixels/2;

        % Check that this is greater than min   
        % <<--
            if yPixelsNew > 430 && xPixelsNew > 550

                % Get the position of the bottom corneer
                %   This will be 1/4 of the screen dimensions
                % <<--
                    yStart = yPixels/4;
                    xStart = xPixels/4;
                % -->>

                % Enlarge the figure
                % <<--
                    set(gcf,'position',[xStart,yStart,xPixelsNew,...
                        yPixelsNew]);
                % -->>

            end
        % -->>
    % -->> 
    
    
    % Display the original RGB image
    % <<--
        imshow(mImage);
        title(['Original Image']);
    % -->>
    
    % Display the remaining channels if RGB image
    % <<--
        if nChannels == 3
            
            cChannelText = {'Red','Green','Blue'};
            
            for iChannel = 1:nChannels

                % Select the subplot
                subplot(1,nSubPlots,iChannel+1);

                % Display the channel
    %             imshow(GetChannel(mImage,i));
                imshow(mImage(:,:,iChannel));
                
                title([cChannelText{iChannel} ' Channel']);

            end
        
        end
    % -->>
    
end

function mChannel = GetChannel(mImage,i)

    % Returns the ith colour channel of mImage
    
    % Create mChannel the same dimensions as mImage with zero intensity 
    % in all channels
    % <<-- 
        mChannel = zeros(size(mImage));
    % -->>
    
    % Populate the ith channel
    % <<--
        mChannel(:,:,i) = mImage(:,:,i);
    % -->>

end
    