#ifndef MATRIXOPERATIONS_H
#define MATRIXOPERATIONS_H

#include <cmath>
#include "MyBLAS.h"
#include "MyLapack.h"
#include <string.h>
#include <algorithm>
#define BLAS 1
#define msgWarning  mexErrMsgTxt
#define msg mexPrintf
#include "my_types.h"


using namespace std;

/*
 * Start of matrix manipulation functions
 */
double RealInnerProduct(double *v1, double *v2, unsigned long szVector, unsigned long incx, unsigned long incy);
double Normalize(double *atom, long szSignal, double normAtom);
void CopyElements(double *array1, double *array2, long nElements);
void AddVectors(double *v1, double *v2, double alpha, long szVector, unsigned long incx, unsigned long incy);
void MultiplyMatrixVector(double *m1, double *v1, double *returnVector, long szVector, long n, char chn);
void ScaleVector(double *v1, double alpha, long szVector);
void MoveColumnToEnd( double *m1, long m, long n, long iMove);
void MoveRowToEnd( double *array1, double *array2, long nRows, long nColumns);
void RemoveColumn( double *m1, long iRemove, long m, long n);
double Norm(double *atom, unsigned long szSignal, unsigned long incx);
void AddVectors1(double *v1, double *v2, double alpha, long szVector);
void SwapElements(double *swapA, double *swapB, long nRows);
void OuterProduct(double *matrix, double *vector, double *vAlpha, unsigned long m, unsigned long n);
void ApplyPlaneRotate( double *v1, double *v2, long n, double DC, double DS);
double Norm2( double *mIn, unsigned long m, unsigned long n);
void KronAtom(double *vX, double *vY, unsigned long szSpaceX, unsigned long szSpaceY, double *vXY);
bool vectorsEqual(unsigned long *v1,unsigned long *v2,unsigned long n, bool sorted);
unsigned long MaxAbs(double *v, unsigned long m);
void AllocateElements(double *v, int alpha, ulng n);
void MatrixMultiplyABC( 
        double *m1, 
        double *m2, 
        double *m3, 
        double *mReturn,
        ulng rowsM1, 
        ulng columnsM1, 
        ulng rowsM2,
        ulng columnsM2,
        char chn1, 
        char chn2,
        char chn3);
ulng MaxAbsDiag(double *v, ulng nRows);
void MatrixMultiply( double *m1, double *m2, double *mReturn, int m, int n, 
        int k, char chn1, char chn2);
void Outer(double *matrix, double *vector, double *vAlpha, unsigned long m,
        unsigned long n, double scale);
double VectorMatrixVector(
        double *vColumns, 
        double *m, 
        double *vRows, 
        ulng nRows,
        ulng nColumns);
void VABS(double *v,ulng nElements);



double Norm2( double *mIn, unsigned long m, unsigned long n)
{
	double *mInCopy = new double[m*n];
	double *mSingular = new double[m];
	CopyElements(mIn,mInCopy,m*n);
	SVDLapack(mInCopy, mSingular, m, n);

	double norm = mSingular[0];

	delete[] mInCopy;
    delete[] mSingular;
	mInCopy = 0;
	mSingular = 0;

	return norm;
}

void OuterProduct(double *matrix, double *vector, double *vAlpha, unsigned long m, unsigned long n)
{


	#ifdef BLAS    
        OuterBLAS(matrix,vector,vAlpha,-1,m,n);
    #else
                
        for ( unsigned long j = 0; j < n ; j++ )
        {      
            /* alpha = (beta*D(:,k)')/nork; */     
            //alpha = RealInnerProduct(&mBiorthogonal[j*szSignal],vNewAtom,szSignal)/normAtom;
            for ( unsigned long i = 0; i < m; i++ )
            {	          	 	  
                matrix[(j*m)+i] = matrix[(j*m)+i] - vAlpha[j]*vector[i];                
            }            

        }                
    
    #endif
}

void Outer(double *matrix, double *v1, double *v2, unsigned long m,
        unsigned long n, double scale)
{


	#ifdef BLAS    
        OuterBLAS(matrix,v1,v2,scale,m,n);
    #else
        
        
        for ( unsigned long j = 0; j < n ; j++ )
        {      
            /* alpha = (beta*D(:,k)')/nork; */     
            //alpha = RealInnerProduct(&mBiorthogonal[j*szSignal],vNewAtom,szSignal)/normAtom;
            for ( unsigned long i = 0; i < m; i++ )
            {	          	 	  
                matrix[(j*m)+i] =  scale*v2[j]*v1[i];                
            }            

        }                
    
    #endif
}
    
double RealInnerProduct(double *v1, double *v2, unsigned long szVector, 
        unsigned long incx = 1, unsigned long incy = 1)
{
    double innerProduct = 0;
#ifdef BLAS
        innerProduct = InnerProductBLAS(v1, v2, szVector, incx, incy);
#else
        if ((incx == 1) && (incy == 1))
        {
            for ( unsigned long i = 0; i < szVector; i++)
            {
                innerProduct += v1[i]*v2[i];  
            }
        }else if ((incx > 1) && (incy == 1))
        {
            for ( unsigned long i = 0; i < szVector; i++)
            {
                innerProduct += v1[i*incx]*v2[i];  
            }
        }else if ((incx == 1) && (incy > 1))
        {
            for ( unsigned long i = 0; i < szVector; i++)
            {
                innerProduct += v1[i]*v2[i*incy];  
            }
		}else if ((incx > 1) && (incy > 1))
        {
            for ( unsigned long i = 0; i < szVector; i++)
            {
                innerProduct += v1[i*incx]*v2[i*incy];  
            }
        }
#endif
    return innerProduct;
}

double Norm(double *atom, unsigned long szSignal, unsigned long incx = 1)
{
    double normAtom = 0;
#ifdef BLAS
            normAtom = CalculateNormBLAS(atom,szSignal,incx);
#else            
            normAtom = sqrt(RealInnerProduct(atom,atom,szSignal,incx, incx));
#endif
    return normAtom;    
}

double Normalize(double *atom, long szSignal, double normAtom)
{   
    if ( normAtom == 0)
    {
        normAtom = Norm(atom,szSignal);
    }

    ScaleVector(atom,1/normAtom,szSignal);
        
    return normAtom;
}

void MultiplyMatrixVector(double *m1, double *v1, double *returnVector, 
        long m, long n, char chn)
{
#ifdef BLAS
        MatrixVectorBLAS(m1, v1, returnVector, m, n, chn);
#else
        if( chn == 'T' )
        {
            for ( long i = 0; i < n ; i++)
            {   
                returnVector[i] = RealInnerProduct(&m1[(i*m)],v1,m);
            }
        }else{            
            for ( long i = 0; i < m ; i++)
            {
                returnVector[i] = 0;
                //mexPrintf("%f %f\n",m1[(j*n)], v1[0]);
                for ( long j = 0; j < n; j++ )
                {
                    //mexPrintf("%f %f\n",m1[(j*m)+i], v1[j]);
                    returnVector[i] = returnVector[i] + m1[(j*m)+i]*v1[j];
                }
            }
        }
#endif
}
              
void AddVectors(double *v1, double *v2, double alpha, long szVector, 
        unsigned long incx = 1, unsigned long incy = 1)
{
    // v1 = v1 + alpha*v2
#ifdef BLAS
        AddVectorsBLAS(v2, v1, alpha, szVector, incy, incx);
#else
        for ( long i = 0; i < szVector; i++ )
        {
            v1[i*incx] = alpha*v2[i*incy] + v1[i*incx];
        }
#endif
}

void AddVectors1(double *v1, double *v2, double alpha, long szVector)
{
    for ( long i = 0; i < szVector; i++ )
    {
        v1[i] = alpha*v1[i] + v2[i];
    }         
}

void ScaleVector(double *v1, double alpha, long szVector)
{
    
#ifdef BLAS
        ScaleVectorBLAS(v1,alpha, szVector);
#else
        for ( long i = 0; i < szVector; i++)
        {
            v1[i] = v1[i]*alpha;
        }
#endif
}

void CopyElements(double *array1, double *array2, long nElements)
{    
    size_t szCopy;
    szCopy = sizeof(double)*nElements;
    memcpy(array2, array1, szCopy);

}
 
void MoveRowToEnd( double *array1, double *array2, long nRows, long nColumns)
{
    /* Moves a row of a matrix to the end.
     * Because we are using the BLAS libraries our matrices are stored in
     * column-major order, therefore we can't use memcpy.
     */
    /* Could have just altered incx to be szSpace, ?Would this be slower? */
    double tmp;
    for( long j = 0; j < nColumns; j++ )
    {
        tmp = array2[j*nColumns];
        //mexPrintf("tmp-%f\n",tmp);
        for( long i = 0; i < nRows; i++ )
        {
            //mexPrintf("array2-%f array1-%f\n" ,array2[j*nColumns+i], array1[j*nColumns+i]);
            array2[j*nColumns+i] = array1[j*nColumns+i];
        }
        //mexPrintf("%i\n",i);
        array1[j*nColumns+(nRows-1)] = tmp;
         
    }    

}

void MoveColumnToEnd( double *m1, long m, long n, long iMove)
{
    /* Declare dynamic arrays remember to delete[] after use */
    long nCopy = (n-(iMove))*m;
    double *mTemp;   
    mTemp = new double[nCopy];
    
    CopyElements(&m1[m*iMove], mTemp, nCopy);
    /* Move the block first */
    CopyElements(&mTemp[m],&m1[m*iMove],nCopy - m);
    /* Now move the vector to the end */
    CopyElements(mTemp,&m1[m*(n-1)],m);
    
    /* Free the memory */
    delete[] mTemp;
    mTemp = 0; // For safety  

}

void RemoveColumn( double *m1, long iRemove, long m, long n)
{
	/* If we are removing the last column then do nothing */
	if (iRemove+1 != n)
	{
		 /* Declare dynamic arrays remember to delete[] after use */
		long nCopy = (n-(iRemove+1))*m;
		double *mTemp;   
		mTemp = new double[nCopy];
	   
		/* Move the block as if using a buffer */
		CopyElements(&m1[m*(iRemove+1)], mTemp, nCopy);
		/* Move the block*/
		CopyElements(mTemp,&m1[m*iRemove],nCopy);
		/* Free the memory */
		delete [] mTemp;
		mTemp = 0; // For safety    
	}
}

void PlaneRotate( double *DA, double *DB, double *DC, double *DS)
{
//if ( abs(*DA**DA - *DB**DB) < 1e-16)
//{
//	msg("Planner Rotate may be incorrect as abs(*DA**DA - *DB**DB) < 1e-16\n");
//}

#ifdef BLAS
        drotg(DA,DB,DC,DS);
#else
        double R = sqrt(*DA**DA + *DB**DB);
        /* Unstable */
        *DC = *DA/R;
        *DS = *DB/R;

        *DA = R;
        *DB = 0;
#endif  
}

void SwapElements(double *swapA, double *swapB, long nRows)
{
#ifdef BLAS
        SwapElementsBLAS(swapA, swapB, nRows);
#else     
        double swappedElement;
        for ( int i = 0; i < nRows; i++ )
        {
            swappedElement = swapB[i];
            swapB[i] = swapA[i];
            swapA[i] = swappedElement;
        }
#endif
}



void ApplyPlaneRotate( double *v1, double *v2, long n, double DC, double DS)
{
#ifdef BLAS
	ApplyPlaneRotateBLAS( v1, v2, n, &DC, &DS);
#else
	double t1;
	double t2;
	for ( long i = 0; i < n ; i++ )
	{
		t1 = v1[i];
		t2 = v2[i];
		v1[i] = DC*t1+DS*t2;
		v2[i] = -DS*t1+DC*t2;
	}
#endif
}

//void MatrixMultiply( double *m1, double *m2, double *mReturn, int m, int n, int k, char chn1, char chn2)
//{
//	// m is the number of rows of m1
//	// n is the number of columns of m1
//	// k is the number of columns of mReturn
//	#ifdef BLAS
//		MatrixMultiplyBLAS(m1, m2, mReturn, m, n, k, chn1, chn2);
//	#else
//		if ( (chn1 == 'N') && (chn2 == 'N') )
//		{
//			for ( long i = 0; i < m; i ++ )
//			{
//				for ( long j = 0; j < k; j++ )
//				{
//					mReturn[(j*m)+i] = 0;
//					for ( long l = 0; l < n; l++ )
//					{
//						mReturn[(j*m)+i] = mReturn[(j*m)+i] + m1[(l*m)+i]*m2[(j*n)+l];
//					}
//				}
//			}
//		}else if ( (chn1 == 'T') && (chn2 == 'N') )
//		{
//			for ( long i = 0; i < n; i ++ )
//			{
//				for ( long j = 0; j < k; j++ )
//				{
//					mReturn[(j*n)+i] = 0;
//					for ( long l = 0; l < m; l++ )
//					{
//						mReturn[(j*n)+i] = mReturn[(j*n)+i] + m1[(i*m)+l]*m2[(j*m)+l];
//					}
//				}
//			}
//		}else if ( (chn1 == 'N') && (chn2 == 'T') )
//		{
//			for ( long i = 0; i < m; i ++ )
//			{
//				for ( long j = 0; j < k; j++ )
//				{
//					mReturn[(j*m)+i] = 0;
//					for ( long l = 0; l < n; l++ )
//					{
//						mReturn[(j*m)+i] = mReturn[(j*m)+i] + m1[(l*m)+i]*m2[(l*n)+j];
//					}
//				}
//			}
//		}else if ( (chn1 == 'T') && (chn2 == 'T') )
//		{
//			for ( long i = 0; i < n; i ++ )
//			{
//				for ( long j = 0; j < k; j++ )
//				{
//					mReturn[(j*n)+i] = 0;
//					for ( long l = 0; l < m; l++ )
//					{
//						mReturn[(j*n)+i] = mReturn[(j*n)+i] + m1[(i*m)+l]*m2[(l*m)+j];
//					}
//				}
//			}
//		}
//	#endif
//}

void MatrixMultiply( double *m1, double *m2, double *mReturn, int m, int n, int k,
        char chn1, char chn2)
{
	// m is the number of rows of m1
	// n is the number of columns of m1
	// k is the number of columns of mReturn
	#ifdef BLAS
		MatrixMultiplyBLAS(m1, m2, mReturn, m, n, k, chn1, chn2);
	#else
		if ( (chn1 == 'N') && (chn2 == 'N') )
		{
			for ( long i = 0; i < m; i ++ )
			{
				for ( long j = 0; j < k; j++ )
				{
					mReturn[(j*m)+i] = 0;
					for ( long l = 0; l < n; l++ )
					{
						mReturn[(j*m)+i] = mReturn[(j*m)+i] + m1[(l*m)+i]*m2[(j*n)+l];
					}
				}
			}
		}else if ( (chn1 == 'T') && (chn2 == 'N') )
		{
			for ( long i = 0; i < n; i ++ )
			{
				for ( long j = 0; j < k; j++ )
				{
					mReturn[(j*n)+i] = 0;
					for ( long l = 0; l < m; l++ )
					{
						mReturn[(j*n)+i] = mReturn[(j*n)+i] + m1[(i*m)+l]*m2[(j*m)+l];
					}
				}
			}
		}else if ( (chn1 == 'N') && (chn2 == 'T') )
		{
			for ( long i = 0; i < m; i ++ )
			{
				for ( long j = 0; j < k; j++ )
				{
					mReturn[(j*m)+i] = 0;
					for ( long l = 0; l < n; l++ )
					{
						mReturn[(j*m)+i] = mReturn[(j*m)+i] + m1[(l*m)+i]*m2[(l*k)+j];
					}
				}
			}
		}else if ( (chn1 == 'T') && (chn2 == 'T') )
		{
			for ( long i = 0; i < n; i ++ )
			{
				for ( long j = 0; j < k; j++ )
				{
					mReturn[(j*n)+i] = 0;
					for ( long l = 0; l < m; l++ )
					{
						mReturn[(j*n)+i] = mReturn[(j*n)+i] + m1[(i*m)+l]*m2[(j*m)+l];
					}
				}
			}
		}
	#endif
}

void MatrixMultiplyABC( 
        double *m1, 
        double *m2, 
        double *m3, 
        double *mReturn,
        ulng rowsM1, 
        ulng columnsM1, 
        ulng rowsM3,
        ulng columnsM3,
        char chn1, 
        char chn2,
        char chn3)
{
    // rowsM1: # of rows in m1
    // m1Columns: # of columns in m1
    // m3Rows: # of rows in m3
    // m3Columns: # of columns in m3
    
    // Works under the assumption that the dimensions of m2 are determined
    // by the dimentions of m1 and m3
    
    
    /* Adjust dimensions of transposed matrices to get dimension of m2 and
     * number of operations in the calculation.
     */
    ulng rowsM1New = rowsM1;
    ulng columnsM1New = columnsM1;
    ulng rowsM3New = rowsM3;
    ulng columnsM3New = columnsM3;    
    if (chn1 == 'T')
    {
        rowsM1New = columnsM1;
        columnsM1New = rowsM1;
    }
    if (chn3 == 'T')
    {
        rowsM3New = columnsM3;
        columnsM3New = rowsM3;
    }
    
    /* Calculate dimensions of m2, needed to determin multiplication order 
     *  The original dimensions will depend on whether the matrix is 
     * transposed
     * Needed for the dimensions of the returned matrix
     */
    // Rows in M2 determined by columns of new M1
    ulng rowsM2 = columnsM1New;
    // Columns in M2 determined by rows of new M3
    ulng columnsM2 = rowsM3New; 
    // New dimensions if no transposition
    ulng rowsM2New = rowsM2;
    ulng columnsM2New = columnsM2;     
    
    /*  The original dimensions will depend on whether the matrix is 
     * transposed
     */
    if (chn2 == 'T')
    {
        
        rowsM2New = columnsM2;
        // Columns in M2 determined by rows in M3
        columnsM2New = rowsM2;  
    }
        
    /* Determin mult order by calculating the number of operations for each 
     * sequence (AB)C A(BC).
     *   Calculation of matrix chain multiplication not optimized.
     */
    ulng nABopC = (rowsM1New*columnsM1New*columnsM2New) + 
            (rowsM1New*columnsM2New*columnsM3New);
    ulng nAopBC = (rowsM2New*columnsM2New*columnsM3New) + 
            (rowsM1New*columnsM1New*columnsM3New);


    /* Multiply */
    if (nABopC <= nAopBC)
    {
        /* Performing (M1M2)M3 */
        /* Initialize dynamic array to store intermidiate matrix          */
        double *mTemp = new double[rowsM1New*columnsM2New];
        MatrixMultiply(m1, m2, mTemp, rowsM1, columnsM1, columnsM2New, chn1, 
                chn2);
        MatrixMultiply(mTemp, m3, mReturn, rowsM1New, columnsM2New, columnsM3New, 
                'N', chn3); // Temp matrix will not be transposed
        delete [] mTemp;
        mTemp = 0;
    }else{
        /* Performing M1(M2M3) */
        /* Initialize dynamic array to store intermidiate matrix */
        double *mTemp = new double[rowsM2New*columnsM3New];
        MatrixMultiply(m2, m3, mTemp, rowsM2, columnsM2, columnsM3New, chn2, chn3);
        MatrixMultiply(m1,mTemp, mReturn, rowsM1, columnsM1, columnsM3New, 
                chn1,'N'); // Temp matrix will not be transposed
        delete [] mTemp;
        mTemp = 0;
    }
    
}


void KronAtom(double *vX, double *vY, unsigned long szSpaceX, unsigned long szSpaceY, double *vXY)
{
    /* Performs the kronecker product of two vextors */
    double xScaleFactor = 0;
    unsigned long iXY  = 0;
    for (unsigned long iX = 0; iX < szSpaceX; iX++)
    {
        xScaleFactor = vX[iX];     
        CopyElements(vY, &vXY[iXY], szSpaceY);        
        ScaleVector(&vXY[iXY], xScaleFactor, szSpaceY);
        iXY += szSpaceY;
    }    
}

bool vectorsEqual(unsigned long *v1,unsigned long *v2,unsigned long n, bool sorted = 0)
{
    
    if(!sorted)
    {
        sort(v1, v1 + n);  
        sort(v2, v2 + n);
    }
    bool equal = true;
    for (unsigned long i = 0; i < n; i ++)
    {
        if (v1[i] != v2[i])
        {
            equal = false;
        }
    }
    return equal;
}



unsigned long MaxAbs(double *v, unsigned long m)
{
// Future add max loop
	// BLAS function returns index 1-n we want c array index 0-n-1
	return MaxAbsBLAS(v, m) - 1;

}


ulng MaxAbsDiag(double *v, 
        ulng nRows)
{
	// BLAS function returns index 1-n we want c array index 0-n-1
	return MaxAbsDiagBLAS(v, nRows) - 1;

}

void AllocateElements(double *v, int alpha, ulng n)
{
	size_t num = static_cast<size_t>(n*sizeof(double));
	memset(v, alpha, num );
}

double VectorMatrixVector(double *vColumns, double *m, double *vRows, 
        ulng nRows,
        ulng nColumns)
{
    /* Calculates 
     * alpha = vColumns'*m*vRows
     * not optimized for order of caculation when vectors are different 
     * sizes.
    */
     double alpha = 0;
    
    /* Multiply the longest vector first 
     * 1xn x nxm x mx1 = 1x1
     * m > n: (1xn) x (nxm x mx1) = nxm + n operations
     * n > m: (1xn x nxm) x (mx1) = nxm + m operations
     */
    if ( nColumns >= nRows)
    {
        /* m >= n: (1xn) x (nxm x mx1) = nxm + n operations*/
        
        /* vTemp = m*vRows:- nRowsxnColumnsxnColumnsx1 = nRowsx1*/
        double *vTemp = new double[nRows];    
        MultiplyMatrixVector(m, vRows, vTemp, nRows, nColumns, 'N');
        
        /* alpha = <vColumns,vTemp> */
        alpha = RealInnerProduct(vColumns, vTemp, nRows);
        
        /* Free dynamic memory */
        delete [] vTemp;
        vTemp = 0;
        
    }else{
        
        /* n > m: (1xn x nxm) x (mx1) = nxm + m operations*/
        
        /* vTemp = m'*vColumns:- nColumnsxnRows x nRowsx1 = nColumnsx1 */
        double *vTemp = new double[nColumns];
        MultiplyMatrixVector(m, vColumns, vTemp, nRows, nColumns, 'T');
        
        /* alpha = <vRows,vTemp> */
        alpha = RealInnerProduct(vRows, vTemp, nColumns);        
        
        /* Free dynamic memory */
        delete [] vTemp;
        vTemp = 0;
        
    }

    return alpha;
  
}


void VABS(double *v,ulng nElements)
{
    
    /* Replaces each element of the vector v with its absolute value */
    for( ulng i = 0; i < nElements; i++ )
    {
        v[i] = v[i] < 0 ? -v[i] : v[i];
    }

}
#endif
