/* 
 * MATLAB Code
    % Control parameters
         % Threshold for coefficients
            tol2=1e-10;   %0.0001  %1e-5
        % - - > >
        % Maximal number of function in sub-dictionary
            maxIter=min(No,nSepAtoms);
        % - - > >
    % - - > > #

    % Initialize loop structures
        % Initialize to max possible size
        % Use coder.nullcopy(zeros(1,N));
            vIChosenC = zeros(1,maxIter);
            vIChosenR = zeros(1,maxIter);
            vISepAtom = zeros(1,maxIter);
            mOrth = zeros(nPixelsPlain,maxIter);
            mAtom = zeros(nPixelsPlain,maxIter);
            mBiorth = zeros(nPixelsPlain,maxIter);
            vNormOrth = zeros(1,maxIter);
        % - - > >

        mRemove = zeros(nRows,nColumns,nColours);

        % Vector to reduce execution time used for calculating mRemove
            mImageVec = reshape(mImage,nPixelsPlain,nColours);
        % - - > >
    % - - > > #

    % Process initial indexes if provided
        numind=numel(vIndC); % Assume both provided
        if numind
            % Validate initial indexes
            %   Testing boundries is quicker than individual indexes
                ValInd(vIndC,1,nColumnAtoms);
                ValInd(vIndR,1,nRowAtoms);
            % - - > >

            % Assign to return vectors
                vIChosenC(1:numind) = vIndC(1:numind);
                vIChosenR(1:numind) = vIndR(1:numind);

                % Calculate seperable atoms index
                    vISepAtom(1:numind) = sub2ind([nColumnAtoms,nRowAtoms],vIndC,...
                        vIndR);
                % - - > > 
            % - - > >
        end
    % - - > > #



    % In loop vars
        nReOrth = 1;
        mResidule = mImage;
        vWeightsNorm = vWeights;
    % - - > > #

    %===============================
    % Main algorithm: at kth iteration
    %===============================
    for iIter = 1:maxIter    
        if iIter <= numind 

            iCAtom = vIChosenC(iIter);
            iRAtom = vIChosenR(iIter);

        else

            % Calculate inner products for each plain
            %   Cannot use average due to subativity of abs
                cc=zeros(nColumnAtoms,nRowAtoms);
                for iColour=1:nColours;
                    cc = cc + vWeights(iColour)*abs(mColumnDict'*...
                        mResidule(:,:,iColour)*mRowDict);
                end
            % - - > >

            % Locate maximum inner product
            %   Get indexes in local dicitonaries and index of seperable atom.
                [maxCoeff iSepAtom] = max(cc(:));
                iRAtom = floor(iSepAtom/nColumnAtoms)+1;            
                iCAtom = iSepAtom - ((iRAtom-1)*nColumnAtoms);
            % - - > >

            % Stopping criterion (coefficient)
            if maxCoeff < tol2 
                iIter = iIter - 1;
                 fprintf('Stopped, max(|<f,q>|/||q||)<= tol2=%g.\n',tol2);
                break;
            else
                % Store chosen atoms
                    vIChosenC(iIter) = iCAtom;
                    vIChosenR(iIter) = iRAtom;
                    vISepAtom(iIter) = iSepAtom;
                % - - > >
            end
        % - - > >
        end  

        % Form seperable atom
        %   Outer product should be faster than kron
            newAtom = reshape(mColumnDict(:,iCAtom)*mRowDict(:,iRAtom)',...
                nPixelsPlain,1);
        % - - > > #

        % Stored for calculation of biorthogonal atoms    
            mAtom(:,iIter) = newAtom;
        % - - > > #

        % Orthogonal atoms step
            % Start building next orthogonal atom
                mOrth(:,iIter) = newAtom;
            % - - > >

            if iIter > 1

                % Orthogonalization of Q(:,k)  w.r.t Q(:,1),..., Q(:,k-1) 
                %   Again use matrix mult for speed - slow in MATLAB fast in c
                    for zi=1:nReOrth+1
                        mOrth(:,iIter) = mOrth(:,iIter) - (mOrth(:,1:iIter-1)*...
                            (mOrth(:,1:iIter-1)'*mOrth(:,iIter)));
                    end
                % - - > >            
            end

            % Normalize orthogonal atom
                normOrth = norm(mOrth(:,iIter));
                mOrth(:,iIter) = mOrth(:,iIter)/normOrth;
                % Required for calculating biorthogonal atoms
                    vNormOrth(iIter) = normOrth;
                % - - > >
            % - - > >
        % - - > > #

        % Calculate component of residule in mOrth
            for iColour = 1:nColours
                mRemove(:,:,iColour) = reshape( (mImageVec(:,iColour)' * ...
                    mOrth(:,iIter)) * mOrth(:,iIter) ,nRows,nColumns );
            end 
        % - - > > #

        % Update the residule
            mResidule = mResidule - mRemove;
        % - - > > #    

        % Update the norm of the residue
            normResidule = vWeightsNorm*reshape(sqrt(sum(sum(mResidule.^2,1),2)),...
                nColours,1);
        % - - > > #

        % Stopping criterion (distance)
        if all(tol~= 0) && all(normResidule < tol)
            break;
        end
    end

    % Calculate biorth here for speed
        mBiorth(:,1) = mOrth(:,1)/vNormOrth(1);
        for i = 2:iIter
            mBiorth(:,1:i-1) = mBiorth(:,1:i-1)-mOrth(:,i)*((mAtom(:,i)'...
                *mBiorth(:,1:i-1))/vNormOrth(i));
            mBiorth(:,i) = mOrth(:,i)/vNormOrth(i);
        end
    % - - > > #

    % Calculate the coefficients
        mCoeff = mImageVec'*mBiorth;
    % - - > >

    % Calculate the approximation
        mApprox = mImage - mResidule;
    % - - > > #

    % Reduce the size of large arrays
        vIChosenC = vIChosenC(1:iIter);
        vIChosenR = vIChosenR(1:iIter);
        mCoeff = mCoeff(:,1:iIter);
        mOrth = mOrth(:,1:iIter);
        mBiorth = mBiorth(:,1:iIter);
        vISepAtom = vISepAtom(1:iIter);
% - - > > #
 * Sub functions
 function ValInd(vInd,minInd,maxInd)

    % Use max and min
        if min(vInd) < minInd  || max(vInd) > maxInd
            error('Demanded index is out of dictionary');
        end
    % - - > > #
 */
#ifndef output
#define output 1
#endif

#ifndef linux
#define linux 0
#endif

#if linux
#include <time.h>
#include <sys/time.h>
#endif

//#include "PursuitFunctions.h"
#include "pursuit_functions.h"
//#include "MatrixOperations.h"
#include "matrix_operations.h"
#include "OMPMlSep_Funs.h"
#include "OMP_Funs.h"
#include "mex.h"
#define msg mexPrintf
#include <algorithm>
//using namespace std;

void OMPMlSep(dbl *mImage,
        dbl *mDictColumns,
        dbl *mDictRows,
        dbl tolApprox,
        dbl *vWeights,
        ulng maxNAtoms,
        dbl *vIndDictColumns,
        dbl *vIndDictRows,
        dbl *mApprox,
        dbl *vIColumnAtoms,
        dbl *vIRowAtoms,
        dbl *mBiorth,
        dbl *mCoeff,
        dbl *mOrth,
        dbl *vISepAtoms,
        ulng &nChosen,
        ulng nInitialIndexes,
        ulng nRows,
        ulng nColumns,
        ulng nColours,
        ulng nAtomsColumn,
        ulng nAtomsRow)
{
    const char name[] = "OMPMlSep";
    
    #if output
        #if linux
            /* tic; */
            timeval start;
            gettimeofday(&start, 0);
        #endif
    #endif
                
        
    /* MATLAB
     % Control parameters
         % Threshold for coefficients
            tol2=1e-10;   %0.0001  %1e-5
        % - - > >
     */
     dbl tolCoeff = 1e-10;
        
     /* MATLAB  
        % Maximal number of function in sub-dictionary
            maxIter=min(No,nSepAtoms);
        % - - > >
    % - - > > #
     */
    ulng nPixelsPlain = nRows*nColumns;      //? where to define?????
        // Used with maxNAtoms nAtomsColumn and nAtomsRow?? 

	/* Extra validation */
	ulng nPossibleAtoms = std::min(nPixelsPlain,nAtomsColumn*nAtomsRow);
	if (maxNAtoms< 0  || maxNAtoms > nPossibleAtoms)
    {

        msg("The minimum number of atoms %i must be greater than 0 and less than %i!\n",maxNAtoms,nPossibleAtoms);
		mexErrMsgTxt("Error!");
    }else if (maxNAtoms != 0)
    {          
        maxNAtoms = std::min(nPossibleAtoms, maxNAtoms);
    }

    /* MATALB
    % Process initial indexes if provided
        numind=numel(vIndC); % Assume both provided
        if numind
            % Validate initial indexes
            %   Testing boundries is quicker than individual indexes
                ValInd(vIndC,1,nColumnAtoms);
                ValInd(vIndR,1,nRowAtoms);
            % - - > >
     */
    if(nInitialIndexes != 0)
    {
        // ValidateIndex throws error
        ValidateIndex(vIndDictColumns, nAtomsColumn, nInitialIndexes);
        ValidateIndex(vIndDictRows, nAtomsRow, nInitialIndexes);        
   
        /* MATLAB
                % Assign to return vectors
                    vIChosenC(1:numind) = vIndC(1:numind);
                    vIChosenR(1:numind) = vIndR(1:numind);
         */
        CopyDbl(vIndDictColumns,vIColumnAtoms,nInitialIndexes);
        CopyDbl(vIndDictRows,vIRowAtoms,nInitialIndexes);
    
        /* MATLAB
                    % Calculate seperable atoms index
                        vISepAtom(1:numind) = sub2ind([nColumnAtoms,nRowAtoms],vIndC,...
                            vIndR);
                    % - - > > 
                % - - > >
            end
         % - - > > #      
         */
        // Pass the smaller vectors
        CalcSepIndex(vISepAtoms,vIndDictColumns,vIndDictRows,nAtomsColumn,
                nInitialIndexes);
    }
 
    /* MATLAB
    % Initialize loop structures
        % Initialize to max possible size
            vNormOrth = zeros(1,maxIter);
			mAtom = zeros(nPixelsPlain,maxIter);
        % - - > >
    % - - > > #   
     */
     dbl *vNormOrth = new dbl[maxNAtoms];
	 dbl *mAtom = new dbl[maxNAtoms*nPixelsPlain];
            
    /* MATLAB
    % In loop vars
        mResidule = mImage;
    % - - > > #
     */
	 ulng nPixels = nPixelsPlain*nColours;
    dbl *mResidule = new dbl[nPixels];
    
    /* Pre weight the residule
     *  Saves applying weights at every iteration and for every norm cacluation
     */
    // Copy mImage to the mResidule
    CopyDbl(mImage,mResidule,nPixels);
    
    // Apply weights if not constant
	if (vWeights[0] != 1.0)
	{
		ulng iStartPlain = 0;
		for (ulng iColour = 0; iColour < nColours; iColour++)
		{
			Matrix::Scale(&mResidule[iStartPlain],vWeights[iColour],nPixelsPlain);
        
			iStartPlain += nPixelsPlain;
		}
	}

	    /* Store the indexes of the seperable atoms
         *  Only required in the loop keep local
         */
        ulng iCAtom = 0;
        ulng iRAtom = 0;
 
		    ulng iSepAtom = 0;
            dbl newCoefficient = 0;

			dbl ssResidule = 0;


    /* MATLAB
    for iIter = 1:maxIter
     */
			ulng iIter = 0;
    for ( iIter = 0; iIter < maxNAtoms; iIter++)
    {

        
        /* MATLAB
        if iIter <= numind
         */
        if (iIter < nInitialIndexes)
        {
            
            /* MATLAB
            iCAtom = vIChosenC(iIter);
            iRAtom = vIChosenR(iIter);
             */
            // Copy from the shorter array -1 for c index
            iCAtom = static_cast<ulng>(vIndDictColumns[iIter]) - 1;
            iRAtom = static_cast<ulng>(vIndDictRows[iIter]) - 1;
            
        }else{
        
            /* MATLAB
            % Calculate inner products for each plain
            %   Cannot use average due to subativity of abs
                cc=zeros(nColumnAtoms,nRowAtoms);
                for iColour=1:nColours;
                    cc = cc + vWeights(iColour)*abs(mColumnDict'*...
                        mResidule(:,:,iColour)*mRowDict);
                end
            % - - > >
             */           

            
            OMPMlSepN::ChooseAtom(mDictColumns,mDictRows,mResidule,
                nRows,nColumns,nAtomsColumn,nAtomsRow,nColours,newCoefficient,
                iSepAtom,iCAtom,iRAtom);

			/* MATLAB
			% Stopping criterion (coefficient)
			if maxCoeff < tol2 
				iIter = iIter - 1;
				fprintf('Stopped, max(|<f,q>|/||q||)<= tol2=%g.\n',tol2);
				break;
			else
				% Store chosen atoms
					vIChosenC(iIter) = iCAtom;
					vIChosenR(iIter) = iRAtom;
					vISepAtom(iIter) = iSepAtom;
				% - - > >
			end
			*/
			// newCoefficient is already an absolute value
			    if ( newCoefficient < tolCoeff )
                {
                    #if output
                        msg("%s stopped, max(|<f,D>|)<= tol2=%g.\n",
                            name,tolCoeff);
                    #endif 
                    break;
                }else{
					/* Plus 1 for MATLAB index change to have 1 array*/
					vIColumnAtoms[iIter] = iCAtom + 1;
					vIRowAtoms[iIter] = iRAtom + 1;
					vISepAtoms[iIter] = iSepAtom + 1;
				}            
        }


				/* MATLAB
				% Form seperable atom
				%   Outer product should be faster than kron
					newAtom = reshape(mColumnDict(:,iCAtom)*mRowDict(:,iRAtom)',...
						nPixelsPlain,1);
				% - - > > #
				*/
		dbl *vNewAtom = new dbl[nPixelsPlain]; // Place directly in mAtom??
		//ZeroFillDbl(vNewAtom,nPixelsPlain);
		Matrix::VOuter(vNewAtom,&mDictColumns[iCAtom*nRows],
			&mDictColumns[iRAtom*nColumns], nRows,nColumns);


		/* MATLAB
		% Stored for calculation of biorthogonal atoms    
			mAtom(:,iIter) = newAtom;
		% - - > > #
		*/
		CopyDbl(vNewAtom,&mAtom[iIter*nPixelsPlain] ,nPixelsPlain);


		/* MATLAB
		% Orthogonal atoms step
        % Start building next orthogonal atom
            mOrth(:,iIter) = newAtom;
        % - - > >
		*/
		CopyDbl(vNewAtom,&mOrth[iIter*nPixelsPlain],nPixelsPlain);

		delete []vNewAtom;
		vNewAtom = 0;


		/* MATLAB
        if iIter > 1
		*/
		if (iIter > 0)
		{
			/* MATLAB
            % Orthogonalization of Q(:,k)  w.r.t Q(:,1),..., Q(:,k-1) 
            %   Again use matrix mult for speed - slow in MATLAB fast in c
                for zi=1:nReOrth+1
                    mOrth(:,iIter) = mOrth(:,iIter) - (mOrth(:,1:iIter-1)*...
                        (mOrth(:,1:iIter-1)'*mOrth(:,iIter)));
                end
            % - - > >   
			end
			*/
			for ( ulng iIter1 = 0; iIter1 < 2; iIter1++)
			{
				// Pass the number chosen therefore iIter + 1
				OMPN::Orthogonalize(mOrth,nPixelsPlain,iIter+1);
			}

		}

			/* MATLAB
				% Normalize orthogonal atom
					normOrth = norm(mOrth(:,iIter));
					mOrth(:,iIter) = mOrth(:,iIter)/normOrth;
					% Required for calculating biorthogonal atoms
						vNormOrth(iIter) = normOrth;
					% - - > >
				% - - > >
			% - - > > #

			*/
			vNormOrth[iIter] = Matrix::VNormalize(&mOrth[iIter*nPixelsPlain],
				nPixelsPlain);

		/* MATLAB
		% Calculate component of residule in mOrth
			for iColour = 1:nColours
				mRemove(:,:,iColour) = reshape( (mImageVec(:,iColour)' * ...
					mOrth(:,iIter)) * mOrth(:,iIter) ,nRows,nColumns );
			end 
		% - - > > #

		% Update the residule
			mResidule = mResidule - mRemove;
		% - - > > #  


		% Update the norm of the residue
			normResidule = sum(reshape(sqrt(sum(sum(mResidule.^2,1),2)),...
				nColours,1));
		% - - > > #
		*/
		if (vWeights[0] == 1)
		{
			ssResidule = OMPMlN::UpdateResidule(mResidule,mImage,&mOrth[iIter*
				nPixelsPlain],nColours,nPixelsPlain);
		}else{
			// Pass the weights vector to weight the inner product mImage(:,:,i)'*mOrth(:,iIter);
			ssResidule = OMPMlN::UpdateResiduleW(mResidule,mImage,&mOrth[iIter*
				nPixelsPlain],vWeights,nColours,nPixelsPlain);
		}

		/* MATLAB
		% Stopping criterion (distance)
		if tol~= 0 && normResidule < tol
			break;
		end
		*/
		if ( ssResidule < tolApprox && tolApprox != 0 )
        {
            /* Break so will not increment iIter before exiting the loop */
            //nChosen += 1;
			iIter = iIter + 1;
            break;                          
        }
    }

	nChosen = iIter;

	/* MATLAB
	% Calculate biorth here for speed
    mBiorth(:,1) = mOrth(:,1)/vNormOrth(1);
    for i = 2:iIter
        mBiorth(:,1:i-1) = mBiorth(:,1:i-1)-mOrth(:,i)*((mAtom(:,i)'...
            *mBiorth(:,1:i-1))/vNormOrth(i));
        mBiorth(:,i) = mOrth(:,i)/vNormOrth(i);
    end
	% - - > > #
	*/
	OMPN::BiorthogonalAtoms(mBiorth,mOrth,mAtom,vNormOrth,nPixelsPlain,nChosen);

	delete []vNormOrth;
	vNormOrth = 0;
	delete []mAtom;
	mAtom = 0;

	/* MATLAB
	% Calculate the coefficients
		mImageVec = reshape(mImage,nPixelsPlain,nColours);
		mCoeff = mImageVec'*mBiorth;
	% - - > >
	*/
	Matrix::MMMult(mImage,mBiorth,mCoeff,nPixelsPlain,nColours,nChosen,
		nColours,'T','N');

	/* MATLAB
	% Remove weighting from the residule
		for i = 1:nColours
			mResidule(:,:,i) = mResidule(:,:,i)/vWeights(i);
		end
	% - - > > #

	% Calculate the approximation
		mApprox = mImage - mResidule;
	% - - > > #
	*/
	CopyDbl(mImage,mApprox,nPixels);

	if (vWeights[0] == 1.0)
	{
		Matrix::VAdd(mApprox,mResidule,-1.0,nPixels);
	}else{
		OMPMlN::UpdateApproxW(mApprox,mResidule,nPixelsPlain,
			nColours,vWeights);
	}

	delete []mResidule;
	mResidule = 0;

}
